# Class: pool&#95;update

Pool&#45;wide updates to the host software

## Fields for class: pool&#95;update

|Field               |Type                |Qualifier      |Description                             |
|:-------------------|:-------------------|:--------------|:---------------------------------------|
|`after_apply_guidance`|`update_after_apply_guidance set`|_RO/constructor_|What the client should do after this update has been applied.|
|`enforce_homogeneity`|`bool`              |_RO/constructor_|Flag &#45; if true, all hosts in a pool must apply this update|
|`hosts`             |`host ref set`      |_RO/runtime_   |The hosts that have applied this update.|
|`installation_size` |`int`               |_RO/constructor_|Size of the update in bytes             |
|`key`               |`string`            |_RO/constructor_|GPG key of the update                   |
|`name_description`  |`string`            |_RO/constructor_|a notes field containing human&#45;readable description|
|`name_label`        |`string`            |_RO/constructor_|a human&#45;readable name               |
|`other_config`      |`(string -> string) map`|_RW_           |additional configuration                |
|`uuid`              |`string`            |_RO/runtime_   |Unique identifier/object reference      |
|`vdi`               |`VDI ref`           |_RO/constructor_|VDI the update was uploaded to          |
|`version`           |`string`            |_RO/constructor_|Update version number                   |

## RPCs associated with class: pool&#95;update

### RPC name: add&#95;to&#95;other&#95;config

_Overview:_

Add the given key&#45;value pair to the other&#95;config field of the given pool&#95;update.

_Signature:_

```
void add_to_other_config (session ref session_ref, pool_update ref self, string key, string value)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`pool_update ref`             |`self`                        |reference to the object                 |
|`string`                      |`key`                         |Key to add                              |
|`string`                      |`value`                       |Value to add                            |

_Minimum Role:_ pool-operator

_Return Type:_ `void`

### RPC name: apply

_Overview:_

Apply the selected update to a host

_Signature:_

```
void apply (session ref session_ref, pool_update ref self, host ref host)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`pool_update ref`             |`self`                        |The update to apply                     |
|`host ref`                    |`host`                        |The host to apply the update to.        |

_Minimum Role:_ pool-operator

_Return Type:_ `void`

### RPC name: destroy

_Overview:_

Removes the database entry. Only works on unapplied update.

_Signature:_

```
void destroy (session ref session_ref, pool_update ref self)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`pool_update ref`             |`self`                        |The update to destroy                   |

_Minimum Role:_ pool-operator

_Return Type:_ `void`

### RPC name: get&#95;after&#95;apply&#95;guidance

_Overview:_

Get the after&#95;apply&#95;guidance field of the given pool&#95;update.

_Signature:_

```
update_after_apply_guidance set get_after_apply_guidance (session ref session_ref, pool_update ref self)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`pool_update ref`             |`self`                        |reference to the object                 |

_Minimum Role:_ read-only

_Return Type:_ `update_after_apply_guidance set`

value of the field

### RPC name: get&#95;all

_Overview:_

Return a list of all the pool&#95;updates known to the system.

_Signature:_

```
pool_update ref set get_all (session ref session_ref)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |

_Minimum Role:_ read-only

_Return Type:_ `pool_update ref set`

references to all objects

### RPC name: get&#95;all&#95;records

_Overview:_

Return a map of pool&#95;update references to pool&#95;update records for all pool&#95;updates known to the system.

_Signature:_

```
(pool_update ref -> pool_update record) map get_all_records (session ref session_ref)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |

_Minimum Role:_ read-only

_Return Type:_ `(pool_update ref -> pool_update record) map`

records of all objects

### RPC name: get&#95;by&#95;name&#95;label

_Overview:_

Get all the pool&#95;update instances with the given label.

_Signature:_

```
pool_update ref set get_by_name_label (session ref session_ref, string label)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`string`                      |`label`                       |label of object to return               |

_Minimum Role:_ read-only

_Return Type:_ `pool_update ref set`

references to objects with matching names

### RPC name: get&#95;by&#95;uuid

_Overview:_

Get a reference to the pool&#95;update instance with the specified UUID.

_Signature:_

```
pool_update ref get_by_uuid (session ref session_ref, string uuid)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`string`                      |`uuid`                        |UUID of object to return                |

_Minimum Role:_ read-only

_Return Type:_ `pool_update ref`

reference to the object

### RPC name: get&#95;enforce&#95;homogeneity

_Overview:_

Get the enforce&#95;homogeneity field of the given pool&#95;update.

_Signature:_

```
bool get_enforce_homogeneity (session ref session_ref, pool_update ref self)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`pool_update ref`             |`self`                        |reference to the object                 |

_Minimum Role:_ read-only

_Return Type:_ `bool`

value of the field

### RPC name: get&#95;hosts

_Overview:_

Get the hosts field of the given pool&#95;update.

_Signature:_

```
host ref set get_hosts (session ref session_ref, pool_update ref self)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`pool_update ref`             |`self`                        |reference to the object                 |

_Minimum Role:_ read-only

_Return Type:_ `host ref set`

value of the field

### RPC name: get&#95;installation&#95;size

_Overview:_

Get the installation&#95;size field of the given pool&#95;update.

_Signature:_

```
int get_installation_size (session ref session_ref, pool_update ref self)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`pool_update ref`             |`self`                        |reference to the object                 |

_Minimum Role:_ read-only

_Return Type:_ `int`

value of the field

### RPC name: get&#95;key

_Overview:_

Get the key field of the given pool&#95;update.

_Signature:_

```
string get_key (session ref session_ref, pool_update ref self)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`pool_update ref`             |`self`                        |reference to the object                 |

_Minimum Role:_ read-only

_Return Type:_ `string`

value of the field

### RPC name: get&#95;name&#95;description

_Overview:_

Get the name/description field of the given pool&#95;update.

_Signature:_

```
string get_name_description (session ref session_ref, pool_update ref self)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`pool_update ref`             |`self`                        |reference to the object                 |

_Minimum Role:_ read-only

_Return Type:_ `string`

value of the field

### RPC name: get&#95;name&#95;label

_Overview:_

Get the name/label field of the given pool&#95;update.

_Signature:_

```
string get_name_label (session ref session_ref, pool_update ref self)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`pool_update ref`             |`self`                        |reference to the object                 |

_Minimum Role:_ read-only

_Return Type:_ `string`

value of the field

### RPC name: get&#95;other&#95;config

_Overview:_

Get the other&#95;config field of the given pool&#95;update.

_Signature:_

```
(string -> string) map get_other_config (session ref session_ref, pool_update ref self)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`pool_update ref`             |`self`                        |reference to the object                 |

_Minimum Role:_ read-only

_Return Type:_ `(string -> string) map`

value of the field

### RPC name: get&#95;record

_Overview:_

Get a record containing the current state of the given pool&#95;update.

_Signature:_

```
pool_update record get_record (session ref session_ref, pool_update ref self)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`pool_update ref`             |`self`                        |reference to the object                 |

_Minimum Role:_ read-only

_Return Type:_ `pool_update record`

all fields from the object

### RPC name: get&#95;uuid

_Overview:_

Get the uuid field of the given pool&#95;update.

_Signature:_

```
string get_uuid (session ref session_ref, pool_update ref self)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`pool_update ref`             |`self`                        |reference to the object                 |

_Minimum Role:_ read-only

_Return Type:_ `string`

value of the field

### RPC name: get&#95;vdi

_Overview:_

Get the vdi field of the given pool&#95;update.

_Signature:_

```
VDI ref get_vdi (session ref session_ref, pool_update ref self)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`pool_update ref`             |`self`                        |reference to the object                 |

_Minimum Role:_ read-only

_Return Type:_ `VDI ref`

value of the field

### RPC name: get&#95;version

_Overview:_

Get the version field of the given pool&#95;update.

_Signature:_

```
string get_version (session ref session_ref, pool_update ref self)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`pool_update ref`             |`self`                        |reference to the object                 |

_Minimum Role:_ read-only

_Return Type:_ `string`

value of the field

### RPC name: introduce

_Overview:_

Introduce update VDI

_Signature:_

```
pool_update ref introduce (session ref session_ref, VDI ref vdi)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VDI ref`                     |`vdi`                         |The VDI which contains a software update.|

_Minimum Role:_ pool-operator

_Return Type:_ `pool_update ref`

the introduced pool update

### RPC name: pool&#95;apply

_Overview:_

Apply the selected update to all hosts in the pool

_Signature:_

```
void pool_apply (session ref session_ref, pool_update ref self)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`pool_update ref`             |`self`                        |The update to apply                     |

_Minimum Role:_ pool-operator

_Return Type:_ `void`

### RPC name: pool&#95;clean

_Overview:_

Removes the update's files from all hosts in the pool, but does not revert the update

_Signature:_

```
void pool_clean (session ref session_ref, pool_update ref self)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`pool_update ref`             |`self`                        |The update to clean up                  |

_Minimum Role:_ pool-operator

_Return Type:_ `void`

### RPC name: precheck

_Overview:_

Execute the precheck stage of the selected update on a host

_Signature:_

```
livepatch_status precheck (session ref session_ref, pool_update ref self, host ref host)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`pool_update ref`             |`self`                        |The update whose prechecks will be run  |
|`host ref`                    |`host`                        |The host to run the prechecks on.       |

_Minimum Role:_ pool-operator

_Return Type:_ `livepatch_status`

The precheck pool update

### RPC name: remove&#95;from&#95;other&#95;config

_Overview:_

Remove the given key and its corresponding value from the other&#95;config field of the given pool&#95;update.  If the key is not in that Map, then do nothing.

_Signature:_

```
void remove_from_other_config (session ref session_ref, pool_update ref self, string key)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`pool_update ref`             |`self`                        |reference to the object                 |
|`string`                      |`key`                         |Key to remove                           |

_Minimum Role:_ pool-operator

_Return Type:_ `void`

### RPC name: set&#95;other&#95;config

_Overview:_

Set the other&#95;config field of the given pool&#95;update.

_Signature:_

```
void set_other_config (session ref session_ref, pool_update ref self, (string -> string) map value)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`pool_update ref`             |`self`                        |reference to the object                 |
|`(string -> string) map`      |`value`                       |New value to set                        |

_Minimum Role:_ pool-operator

_Return Type:_ `void`
