# Class: VM

A virtual machine &#40;or 'guest'&#41;.

## Fields for class: VM

|Field               |Type                |Qualifier      |Description                             |
|:-------------------|:-------------------|:--------------|:---------------------------------------|
|`actions_after_crash`|`on_crash_behaviour`|_RO/constructor_|action to take if the guest crashes     |
|`actions_after_reboot`|`on_normal_exit`    |_RW_           |action to take after the guest has rebooted itself|
|`actions_after_shutdown`|`on_normal_exit`    |_RW_           |action to take after the guest has shutdown itself|
|`actions_after_softreboot`|`on_softreboot_behavior`|_RW_           |action to take after soft reboot        |
|`affinity`          |`host ref`          |_RW_           |A host which the VM has some affinity for &#40;or NULL&#41;. This is used as a hint to the start call when it decides where to run the VM. Resource constraints may cause the VM to be started elsewhere.|
|`allowed_operations`|`vm_operations set` |_RO/runtime_   |list of the operations allowed in this state. This list is advisory only and the server state may have changed by the time this field is read by a client.|
|`appliance`         |`VM_appliance ref`  |_RO/constructor_|the appliance to which this VM belongs  |
|`attached_PCIs`     |`PCI ref set`       |_RO/runtime_   |Currently passed&#45;through PCI devices|
|`bios_strings`      |`(string -> string) map`|_RO/runtime_   |BIOS strings                            |
|`blobs`             |`(string -> blob ref) map`|_RO/runtime_   |Binary blobs associated with this VM    |
|`blocked_operations`|`(vm_operations -> string) map`|_RO/constructor_|List of operations which have been explicitly blocked and an error code|
|`children`          |`VM ref set`        |_RO/runtime_   |List pointing to all the children of this VM|
|`consoles`          |`console ref set`   |_RO/runtime_   |virtual console devices                 |
|`crash_dumps`       |`crashdump ref set` |_RO/runtime_   |crash dumps associated with this VM     |
|`current_operations`|`(string -> vm_operations) map`|_RO/runtime_   |links each of the running tasks using this object &#40;by reference&#41; to a current&#95;operation enum which describes the nature of the task.|
|`domain_type`       |`domain_type`       |_RO/constructor_|The type of domain that will be created when the VM is started|
|`domarch`           |`string`            |_RO/runtime_   |Domain architecture &#40;if available, null string otherwise&#41;|
|`domid`             |`int`               |_RO/runtime_   |domain ID &#40;if available, &#45;1 otherwise&#41;|
|`generation_id`     |`string`            |_RO/constructor_|Generation ID of the VM                 |
|`groups`            |`VM_group ref set`  |_RO/runtime_   |VM groups associated with the VM        |
|`guest_metrics`     |`VM_guest_metrics ref`|_RO/runtime_   |metrics associated with the running guest|
|`ha_always_run`     |`bool`              |_RO/constructor_|**Deprecated.** if true then the system will attempt to keep the VM running as much as possible.|
|`ha_restart_priority`|`string`            |_RO/constructor_|has possible values: "best&#45;effort" meaning "try to restart this VM if possible but don't consider the Pool to be overcommitted if this is not possible"; "restart" meaning "this VM should be restarted"; "" meaning "do not try to restart this VM"|
|`hardware_platform_version`|`int`               |_RW_           |The host virtual hardware platform version the VM can run on|
|`has_vendor_device` |`bool`              |_RO/constructor_|When an HVM guest starts, this controls the presence of the emulated C000 PCI device which triggers Windows Update to fetch or update PV drivers.|
|`HVM_boot_params`   |`(string -> string) map`|_RW_           |HVM boot params                         |
|`HVM_boot_policy`   |`string`            |_RO/constructor_|**Deprecated.** HVM boot policy                         |
|`HVM_shadow_multiplier`|`float`             |_RO/constructor_|multiplier applied to the amount of shadow that will be made available to the guest|
|`is_a_snapshot`     |`bool`              |_RO/runtime_   |true if this is a snapshot. Snapshotted VMs can never be started, they are used only for cloning other VMs|
|`is_a_template`     |`bool`              |_RW_           |true if this is a template. Template VMs can never be started, they are used only for cloning other VMs|
|`is_control_domain` |`bool`              |_RO/runtime_   |true if this is a control domain &#40;domain 0 or a driver domain&#41;|
|`is_default_template`|`bool`              |_RO/runtime_   |true if this is a default template. Default template VMs can never be started or migrated, they are used only for cloning other VMs|
|`is_snapshot_from_vmpp`|`bool`              |_RO/constructor_|**Removed.** true if this snapshot was created by the protection policy|
|`is_vmss_snapshot`  |`bool`              |_RO/constructor_|true if this snapshot was created by the snapshot schedule|
|`last_boot_CPU_flags`|`(string -> string) map`|_RO/constructor_|describes the CPU flags on which the VM was last booted|
|`last_booted_record`|`string`            |_RO/constructor_|marshalled value containing VM record at time of last boot|
|`memory_dynamic_max`|`int`               |_RO/constructor_|Dynamic maximum &#40;bytes&#41;         |
|`memory_dynamic_min`|`int`               |_RO/constructor_|Dynamic minimum &#40;bytes&#41;         |
|`memory_overhead`   |`int`               |_RO/runtime_   |Virtualization memory overhead &#40;bytes&#41;.|
|`memory_static_max` |`int`               |_RO/constructor_|Statically&#45;set &#40;i.e. absolute&#41; maximum &#40;bytes&#41;. The value of this field at VM start time acts as a hard limit of the amount of memory a guest can use. New values only take effect on reboot.|
|`memory_static_min` |`int`               |_RO/constructor_|Statically&#45;set &#40;i.e. absolute&#41; mininum &#40;bytes&#41;. The value of this field indicates the least amount of memory this VM can boot with without crashing.|
|`memory_target`     |`int`               |_RO/constructor_|**Deprecated.** Dynamically&#45;set memory target &#40;bytes&#41;. The value of this field indicates the current target for memory available to this VM.|
|`metrics`           |`VM_metrics ref`    |_RO/runtime_   |metrics associated with this VM         |
|`name_description`  |`string`            |_RW_           |a notes field containing human&#45;readable description|
|`name_label`        |`string`            |_RW_           |a human&#45;readable name               |
|`NVRAM`             |`(string -> string) map`|_RO/constructor_|initial value for guest NVRAM &#40;containing UEFI variables, etc&#41;. Cannot be changed while the VM is running|
|`order`             |`int`               |_RO/constructor_|The point in the startup or shutdown sequence at which this VM will be started|
|`other_config`      |`(string -> string) map`|_RW_           |additional configuration                |
|`parent`            |`VM ref`            |_RO/runtime_   |Ref pointing to the parent of this VM   |
|`PCI_bus`           |`string`            |_RW_           |**Deprecated.** PCI bus path for pass&#45;through devices|
|`pending_guidances` |`update_guidances set`|_RO/runtime_   |The set of pending mandatory guidances after applying updates, which must be applied, as otherwise there may be e.g. VM failures|
|`pending_guidances_full`|`update_guidances set`|_RO/runtime_   |The set of pending full guidances after applying updates, which a user should follow to make some updates, e.g. specific hardware drivers or CPU features, fully effective, but the 'average user' doesn't need to|
|`pending_guidances_recommended`|`update_guidances set`|_RO/runtime_   |The set of pending recommended guidances after applying updates, which most users should follow to make the updates effective, but if not followed, will not cause a failure|
|`platform`          |`(string -> string) map`|_RW_           |platform&#45;specific configuration     |
|`power_state`       |`vm_power_state`    |_RO/constructor_|Current power state of the machine      |
|`protection_policy` |`VMPP ref`          |_RO/constructor_|**Deprecated.** Ref pointing to a protection policy for this VM|
|`PV_args`           |`string`            |_RW_           |kernel command&#45;line arguments       |
|`PV_bootloader`     |`string`            |_RW_           |name of or path to bootloader           |
|`PV_bootloader_args`|`string`            |_RW_           |miscellaneous arguments for the bootloader|
|`PV_kernel`         |`string`            |_RW_           |path to the kernel                      |
|`PV_legacy_args`    |`string`            |_RW_           |to make Zurich guests boot              |
|`PV_ramdisk`        |`string`            |_RW_           |path to the initrd                      |
|`recommendations`   |`string`            |_RW_           |An XML specification of recommended values and ranges for properties of this VM|
|`reference_label`   |`string`            |_RO/constructor_|Textual reference to the template used to create a VM. This can be used by clients in need of an immutable reference to the template since the latter's uuid and name&#95;label may change, for example, after a package installation or upgrade.|
|`requires_reboot`   |`bool`              |_RO/runtime_   |Indicates whether a VM requires a reboot in order to update its configuration, e.g. its memory allocation.|
|`resident_on`       |`host ref`          |_RO/runtime_   |the host the VM is currently resident on|
|`scheduled_to_be_resident_on`|`host ref`          |_RO/runtime_   |the host on which the VM is due to be started/resumed/migrated. This acts as a memory reservation indicator|
|`shutdown_delay`    |`int`               |_RO/constructor_|The delay to wait before proceeding to the next order in the shutdown sequence &#40;seconds&#41;|
|`snapshot_info`     |`(string -> string) map`|_RO/runtime_   |Human&#45;readable information concerning this snapshot|
|`snapshot_metadata` |`string`            |_RO/runtime_   |Encoded information about the VM's metadata this is a snapshot of|
|`snapshot_of`       |`VM ref`            |_RO/runtime_   |Ref pointing to the VM this snapshot is of.|
|`snapshot_schedule` |`VMSS ref`          |_RO/constructor_|Ref pointing to a snapshot schedule for this VM|
|`snapshot_time`     |`datetime`          |_RO/runtime_   |Date/time when this snapshot was created.|
|`snapshots`         |`VM ref set`        |_RO/runtime_   |List pointing to all the VM snapshots.  |
|`start_delay`       |`int`               |_RO/constructor_|The delay to wait before proceeding to the next order in the startup sequence &#40;seconds&#41;|
|`suspend_SR`        |`SR ref`            |_RW_           |The SR on which a suspend image is stored|
|`suspend_VDI`       |`VDI ref`           |_RO/constructor_|The VDI that a suspend image is stored on. &#40;Only has meaning if VM is currently suspended&#41;|
|`tags`              |`string set`        |_RW_           |user&#45;specified tags for categorization purposes|
|`transportable_snapshot_id`|`string`            |_RO/runtime_   |Transportable ID of the snapshot VM     |
|`user_version`      |`int`               |_RW_           |Creators of VMs and templates may store version information here.|
|`uuid`              |`string`            |_RO/runtime_   |Unique identifier/object reference      |
|`VBDs`              |`VBD ref set`       |_RO/runtime_   |virtual block devices                   |
|`VCPUs_at_startup`  |`int`               |_RO/constructor_|Boot number of VCPUs                    |
|`VCPUs_max`         |`int`               |_RO/constructor_|Max number of VCPUs                     |
|`VCPUs_params`      |`(string -> string) map`|_RW_           |configuration parameters for the selected VCPU policy|
|`version`           |`int`               |_RO/constructor_|The number of times this VM has been recovered|
|`VGPUs`             |`VGPU ref set`      |_RO/runtime_   |Virtual GPUs                            |
|`VIFs`              |`VIF ref set`       |_RO/runtime_   |virtual network interfaces              |
|`VTPMs`             |`VTPM ref set`      |_RO/runtime_   |virtual TPMs                            |
|`VUSBs`             |`VUSB ref set`      |_RO/runtime_   |virtual usb devices                     |
|`xenstore_data`     |`(string -> string) map`|_RW_           |data to be inserted into the xenstore tree &#40;/local/domain/&lt;domid&gt;/vm&#45;data&#41; after the VM is created.|

## RPCs associated with class: VM

### RPC name: add&#95;tags

_Overview:_

Add the given value to the tags field of the given VM.  If the value is already in that Set, then do nothing.

_Signature:_

```
void add_tags (session ref session_ref, VM ref self, string value)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |reference to the object                 |
|`string`                      |`value`                       |New value to add                        |

_Minimum Role:_ vm-operator

_Return Type:_ `void`

### RPC name: add&#95;to&#95;blocked&#95;operations

_Overview:_

Update list of operations which have been explicitly blocked and an error code

_Signature:_

```
void add_to_blocked_operations (session ref session_ref, VM ref self, vm_operations key, string value)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |The VM                                  |
|`vm_operations`               |`key`                         |Blocked operation                       |
|`string`                      |`value`                       |Error code                              |

_Minimum Role:_ vm-admin

_Return Type:_ `void`

### RPC name: add&#95;to&#95;HVM&#95;boot&#95;params

_Overview:_

Add the given key&#45;value pair to the HVM/boot&#95;params field of the given VM.

_Signature:_

```
void add_to_HVM_boot_params (session ref session_ref, VM ref self, string key, string value)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |reference to the object                 |
|`string`                      |`key`                         |Key to add                              |
|`string`                      |`value`                       |Value to add                            |

_Minimum Role:_ vm-admin

_Return Type:_ `void`

### RPC name: add&#95;to&#95;NVRAM

_Overview:_

_Signature:_

```
void add_to_NVRAM (session ref session_ref, VM ref self, string key, string value)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |The VM                                  |
|`string`                      |`key`                         |The key                                 |
|`string`                      |`value`                       |The value                               |

_Minimum Role:_ vm-admin

_Return Type:_ `void`

### RPC name: add&#95;to&#95;other&#95;config

_Overview:_

Add the given key&#45;value pair to the other&#95;config field of the given VM.

_Signature:_

```
void add_to_other_config (session ref session_ref, VM ref self, string key, string value)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |reference to the object                 |
|`string`                      |`key`                         |Key to add                              |
|`string`                      |`value`                       |Value to add                            |

_Minimum Role:_ vm-admin

_Return Type:_ `void`

### RPC name: add&#95;to&#95;platform

_Overview:_

Add the given key&#45;value pair to the platform field of the given VM.

_Signature:_

```
void add_to_platform (session ref session_ref, VM ref self, string key, string value)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |reference to the object                 |
|`string`                      |`key`                         |Key to add                              |
|`string`                      |`value`                       |Value to add                            |

_Minimum Role:_ vm-admin

_Return Type:_ `void`

### RPC name: add&#95;to&#95;VCPUs&#95;params

_Overview:_

Add the given key&#45;value pair to the VCPUs/params field of the given VM.

_Signature:_

```
void add_to_VCPUs_params (session ref session_ref, VM ref self, string key, string value)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |reference to the object                 |
|`string`                      |`key`                         |Key to add                              |
|`string`                      |`value`                       |Value to add                            |

_Minimum Role:_ vm-admin

_Return Type:_ `void`

### RPC name: add&#95;to&#95;VCPUs&#95;params&#95;live

_Overview:_

Add the given key&#45;value pair to VM.VCPUs&#95;params, and apply that value on the running VM

_Signature:_

```
void add_to_VCPUs_params_live (session ref session_ref, VM ref self, string key, string value)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |The VM                                  |
|`string`                      |`key`                         |The key                                 |
|`string`                      |`value`                       |The value                               |

_Minimum Role:_ vm-admin

_Return Type:_ `void`

### RPC name: add&#95;to&#95;xenstore&#95;data

_Overview:_

Add the given key&#45;value pair to the xenstore&#95;data field of the given VM.

_Signature:_

```
void add_to_xenstore_data (session ref session_ref, VM ref self, string key, string value)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |reference to the object                 |
|`string`                      |`key`                         |Key to add                              |
|`string`                      |`value`                       |Value to add                            |

_Minimum Role:_ vm-admin

_Return Type:_ `void`

### RPC name: assert&#95;agile

_Overview:_

Returns an error if the VM is not considered agile e.g. because it is tied to a resource local to a host

_Signature:_

```
void assert_agile (session ref session_ref, VM ref self)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |The VM                                  |

_Minimum Role:_ read-only

_Return Type:_ `void`

### RPC name: assert&#95;can&#95;be&#95;recovered

_Overview:_

Assert whether all SRs required to recover this VM are available.

_Signature:_

```
void assert_can_be_recovered (session ref session_ref, VM ref self, session ref session_to)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |The VM to recover                       |
|`session ref`                 |`session_to`                  |The session to which the VM is to be recovered.|

_Minimum Role:_ read-only

_Return Type:_ `void`

_Possible Error Codes:_ `VM_IS_PART_OF_AN_APPLIANCE`, `VM_REQUIRES_SR`

### RPC name: assert&#95;can&#95;boot&#95;here

_Overview:_

Returns an error if the VM could not boot on this host for some reason

_Signature:_

```
void assert_can_boot_here (session ref session_ref, VM ref self, host ref host)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |The VM                                  |
|`host ref`                    |`host`                        |The host                                |

_Minimum Role:_ read-only

_Return Type:_ `void`

_Possible Error Codes:_ `HOST_NOT_ENOUGH_FREE_MEMORY`, `HOST_NOT_ENOUGH_PCPUS`, `NETWORK_SRIOV_INSUFFICIENT_CAPACITY`, `HOST_NOT_LIVE`, `HOST_DISABLED`, `HOST_CANNOT_ATTACH_NETWORK`, `VM_HVM_REQUIRED`, `VM_REQUIRES_GPU`, `VM_REQUIRES_IOMMU`, `VM_REQUIRES_NETWORK`, `VM_REQUIRES_SR`, `VM_REQUIRES_VGPU`, `VM_HOST_INCOMPATIBLE_VERSION`, `VM_HOST_INCOMPATIBLE_VIRTUAL_HARDWARE_PLATFORM_VERSION`, `INVALID_VALUE`, `MEMORY_CONSTRAINT_VIOLATION`, `OPERATION_NOT_ALLOWED`, `VALUE_NOT_SUPPORTED`, `VM_INCOMPATIBLE_WITH_THIS_HOST`

### RPC name: assert&#95;can&#95;migrate

_Overview:_

Assert whether a VM can be migrated to the specified destination.

_Signature:_

```
void assert_can_migrate (session ref session_ref, VM ref vm, (string -> string) map dest, bool live, (VDI ref -> SR ref) map vdi_map, (VIF ref -> network ref) map vif_map, (string -> string) map options, (VGPU ref -> GPU_group ref) map vgpu_map)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`vm`                          |The VM                                  |
|`(string -> string) map`      |`dest`                        |The result of a VM.migrate&#95;receive call.|
|`bool`                        |`live`                        |Live migration                          |
|`(VDI ref -> SR ref) map`     |`vdi_map`                     |Map of source VDI to destination SR     |
|`(VIF ref -> network ref) map`|`vif_map`                     |Map of source VIF to destination network|
|`(string -> string) map`      |`options`                     |Other parameters                        |
|`(VGPU ref -> GPU_group ref) map`|`vgpu_map`                    |Map of source vGPU to destination GPU group|

_Minimum Role:_ vm-power-admin

_Return Type:_ `void`

_Possible Error Codes:_ `LICENCE_RESTRICTION`

### RPC name: assert&#95;operation&#95;valid

_Overview:_

Check to see whether this operation is acceptable in the current state of the system, raising an error if the operation is invalid for some reason

_Signature:_

```
void assert_operation_valid (session ref session_ref, VM ref self, vm_operations op)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |reference to the object                 |
|`vm_operations`               |`op`                          |proposed operation                      |

_Minimum Role:_ read-only

_Return Type:_ `void`

### RPC name: call&#95;plugin

_Overview:_

Call an API plugin on this vm

_Signature:_

```
string call_plugin (session ref session_ref, VM ref vm, string plugin, string fn, (string -> string) map args)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`vm`                          |The vm                                  |
|`string`                      |`plugin`                      |The name of the plugin                  |
|`string`                      |`fn`                          |The name of the function within the plugin|
|`(string -> string) map`      |`args`                        |Arguments for the function              |

_Minimum Role:_ vm-operator

_Return Type:_ `string`

Result from the plugin

### RPC name: checkpoint

_Overview:_

Checkpoints the specified VM, making a new VM. Checkpoint automatically exploits the capabilities of the underlying storage repository in which the VM's disk images are stored &#40;e.g. Copy on Write&#41; and saves the memory image as well.

_Signature:_

```
VM ref checkpoint (session ref session_ref, VM ref vm, string new_name)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`vm`                          |The VM to be checkpointed               |
|`string`                      |`new_name`                    |The name of the checkpointed VM         |

_Minimum Role:_ vm-power-admin

_Return Type:_ `VM ref`

The reference of the newly created VM.

_Possible Error Codes:_ `VM_BAD_POWER_STATE`, `SR_FULL`, `OPERATION_NOT_ALLOWED`, `VM_CHECKPOINT_SUSPEND_FAILED`, `VM_CHECKPOINT_RESUME_FAILED`

### RPC name: clean&#95;reboot

_Overview:_

Attempt to cleanly shutdown the specified VM &#40;Note: this may not be supported&#45;&#45;&#45;e.g. if a guest agent is not installed&#41;. This can only be called when the specified VM is in the Running state.

_Signature:_

```
void clean_reboot (session ref session_ref, VM ref vm)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`vm`                          |The VM to shutdown                      |

_Minimum Role:_ vm-operator

_Return Type:_ `void`

_Possible Error Codes:_ `VM_BAD_POWER_STATE`, `OTHER_OPERATION_IN_PROGRESS`, `OPERATION_NOT_ALLOWED`, `VM_IS_TEMPLATE`

### RPC name: clean&#95;shutdown

_Overview:_

Attempt to cleanly shutdown the specified VM. &#40;Note: this may not be supported&#45;&#45;&#45;e.g. if a guest agent is not installed&#41;. This can only be called when the specified VM is in the Running state.

_Signature:_

```
void clean_shutdown (session ref session_ref, VM ref vm)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`vm`                          |The VM to shutdown                      |

_Minimum Role:_ vm-operator

_Return Type:_ `void`

_Possible Error Codes:_ `VM_BAD_POWER_STATE`, `OTHER_OPERATION_IN_PROGRESS`, `OPERATION_NOT_ALLOWED`, `VM_IS_TEMPLATE`

### RPC name: clone

_Overview:_

Clones the specified VM, making a new VM. Clone automatically exploits the capabilities of the underlying storage repository in which the VM's disk images are stored &#40;e.g. Copy on Write&#41;.   This function can only be called when the VM is in the Halted State.

_Signature:_

```
VM ref clone (session ref session_ref, VM ref vm, string new_name)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`vm`                          |The VM to be cloned                     |
|`string`                      |`new_name`                    |The name of the cloned VM               |

_Minimum Role:_ vm-admin

_Return Type:_ `VM ref`

The reference of the newly created VM.

_Possible Error Codes:_ `VM_BAD_POWER_STATE`, `SR_FULL`, `OPERATION_NOT_ALLOWED`, `LICENCE_RESTRICTION`

### RPC name: compute&#95;memory&#95;overhead

_Overview:_

Computes the virtualization memory overhead of a VM.

_Signature:_

```
int compute_memory_overhead (session ref session_ref, VM ref vm)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`vm`                          |The VM for which to compute the memory overhead|

_Minimum Role:_ read-only

_Return Type:_ `int`

the virtualization memory overhead of the VM.

### RPC name: copy

_Overview:_

Copied the specified VM, making a new VM. Unlike clone, copy does not exploits the capabilities of the underlying storage repository in which the VM's disk images are stored. Instead, copy guarantees that the disk images of the newly created VM will be 'full disks' &#45; i.e. not part of a CoW chain.  This function can only be called when the VM is in the Halted State.

_Signature:_

```
VM ref copy (session ref session_ref, VM ref vm, string new_name, SR ref sr)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`vm`                          |The VM to be copied                     |
|`string`                      |`new_name`                    |The name of the copied VM               |
|`SR ref`                      |`sr`                          |An SR to copy all the VM's disks into &#40;if an invalid reference then it uses the existing SRs&#41;|

_Minimum Role:_ vm-admin

_Return Type:_ `VM ref`

The reference of the newly created VM.

_Possible Error Codes:_ `VM_BAD_POWER_STATE`, `SR_FULL`, `OPERATION_NOT_ALLOWED`, `LICENCE_RESTRICTION`

### RPC name: copy&#95;bios&#95;strings

_Overview:_

Copy the BIOS strings from the given host to this VM

_Signature:_

```
void copy_bios_strings (session ref session_ref, VM ref vm, host ref host)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`vm`                          |The VM to modify                        |
|`host ref`                    |`host`                        |The host to copy the BIOS strings from  |

_Minimum Role:_ vm-admin

_Return Type:_ `void`

### RPC name: create

_Overview:_

NOT RECOMMENDED&#33; VM.clone or VM.copy &#40;or VM.import&#41; is a better choice in almost all situations. The standard way to obtain a new VM is to call VM.clone on a template VM, then call VM.provision on the new clone. Caution: if VM.create is used and then the new VM is attached to a virtual disc that has an operating system already installed, then there is no guarantee that the operating system will boot and run. Any software that calls VM.create on a future version of this API may fail or give unexpected results. For example this could happen if an additional parameter were added to VM.create. VM.create is intended only for use in the automatic creation of the system VM templates. It creates a new VM instance, and returns its handle.

_Signature:_

```
VM ref create (session ref session_ref, VM record args)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM record`                   |`args`                        |All constructor arguments               |

_Minimum Role:_ vm-admin

_Return Type:_ `VM ref`

reference to the newly created object

### RPC name: create&#95;new&#95;blob

_Overview:_

Create a placeholder for a named binary blob of data that is associated with this VM

_Signature:_

```
blob ref create_new_blob (session ref session_ref, VM ref vm, string name, string mime_type, bool public)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`vm`                          |The VM                                  |
|`string`                      |`name`                        |The name associated with the blob       |
|`string`                      |`mime_type`                   |The mime type for the data. Empty string translates to application/octet&#45;stream|
|`bool`                        |`public`                      |True if the blob should be publicly available|

_Minimum Role:_ vm-power-admin

_Return Type:_ `blob ref`

The reference of the blob, needed for populating its data

### RPC name: destroy

_Overview:_

Destroy the specified VM.  The VM is completely removed from the system.  This function can only be called when the VM is in the Halted State.

_Signature:_

```
void destroy (session ref session_ref, VM ref self)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |reference to the object                 |

_Minimum Role:_ vm-admin

_Return Type:_ `void`

### RPC name: forget&#95;data&#95;source&#95;archives

_Overview:_

Forget the recorded statistics related to the specified data source

_Signature:_

```
void forget_data_source_archives (session ref session_ref, VM ref self, string data_source)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |The VM                                  |
|`string`                      |`data_source`                 |The data source whose archives are to be forgotten|

_Minimum Role:_ vm-admin

_Return Type:_ `void`

### RPC name: get&#95;actions&#95;after&#95;crash

_Overview:_

Get the actions/after&#95;crash field of the given VM.

_Signature:_

```
on_crash_behaviour get_actions_after_crash (session ref session_ref, VM ref self)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |reference to the object                 |

_Minimum Role:_ read-only

_Return Type:_ `on_crash_behaviour`

value of the field

### RPC name: get&#95;actions&#95;after&#95;reboot

_Overview:_

Get the actions/after&#95;reboot field of the given VM.

_Signature:_

```
on_normal_exit get_actions_after_reboot (session ref session_ref, VM ref self)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |reference to the object                 |

_Minimum Role:_ read-only

_Return Type:_ `on_normal_exit`

value of the field

### RPC name: get&#95;actions&#95;after&#95;shutdown

_Overview:_

Get the actions/after&#95;shutdown field of the given VM.

_Signature:_

```
on_normal_exit get_actions_after_shutdown (session ref session_ref, VM ref self)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |reference to the object                 |

_Minimum Role:_ read-only

_Return Type:_ `on_normal_exit`

value of the field

### RPC name: get&#95;actions&#95;after&#95;softreboot

_Overview:_

Get the actions/after&#95;softreboot field of the given VM.

_Signature:_

```
on_softreboot_behavior get_actions_after_softreboot (session ref session_ref, VM ref self)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |reference to the object                 |

_Minimum Role:_ read-only

_Return Type:_ `on_softreboot_behavior`

value of the field

### RPC name: get&#95;affinity

_Overview:_

Get the affinity field of the given VM.

_Signature:_

```
host ref get_affinity (session ref session_ref, VM ref self)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |reference to the object                 |

_Minimum Role:_ read-only

_Return Type:_ `host ref`

value of the field

### RPC name: get&#95;all

_Overview:_

Return a list of all the VMs known to the system.

_Signature:_

```
VM ref set get_all (session ref session_ref)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |

_Minimum Role:_ read-only

_Return Type:_ `VM ref set`

references to all objects

### RPC name: get&#95;all&#95;records

_Overview:_

Return a map of VM references to VM records for all VMs known to the system.

_Signature:_

```
(VM ref -> VM record) map get_all_records (session ref session_ref)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |

_Minimum Role:_ read-only

_Return Type:_ `(VM ref -> VM record) map`

records of all objects

### RPC name: get&#95;allowed&#95;operations

_Overview:_

Get the allowed&#95;operations field of the given VM.

_Signature:_

```
vm_operations set get_allowed_operations (session ref session_ref, VM ref self)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |reference to the object                 |

_Minimum Role:_ read-only

_Return Type:_ `vm_operations set`

value of the field

### RPC name: get&#95;allowed&#95;VBD&#95;devices

_Overview:_

Returns a list of the allowed values that a VBD device field can take

_Signature:_

```
string set get_allowed_VBD_devices (session ref session_ref, VM ref vm)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`vm`                          |The VM to query                         |

_Minimum Role:_ read-only

_Return Type:_ `string set`

The allowed values

### RPC name: get&#95;allowed&#95;VIF&#95;devices

_Overview:_

Returns a list of the allowed values that a VIF device field can take

_Signature:_

```
string set get_allowed_VIF_devices (session ref session_ref, VM ref vm)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`vm`                          |The VM to query                         |

_Minimum Role:_ read-only

_Return Type:_ `string set`

The allowed values

### RPC name: get&#95;appliance

_Overview:_

Get the appliance field of the given VM.

_Signature:_

```
VM_appliance ref get_appliance (session ref session_ref, VM ref self)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |reference to the object                 |

_Minimum Role:_ read-only

_Return Type:_ `VM_appliance ref`

value of the field

### RPC name: get&#95;attached&#95;PCIs

_Overview:_

Get the attached&#95;PCIs field of the given VM.

_Signature:_

```
PCI ref set get_attached_PCIs (session ref session_ref, VM ref self)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |reference to the object                 |

_Minimum Role:_ read-only

_Return Type:_ `PCI ref set`

value of the field

### RPC name: get&#95;bios&#95;strings

_Overview:_

Get the bios&#95;strings field of the given VM.

_Signature:_

```
(string -> string) map get_bios_strings (session ref session_ref, VM ref self)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |reference to the object                 |

_Minimum Role:_ read-only

_Return Type:_ `(string -> string) map`

value of the field

### RPC name: get&#95;blobs

_Overview:_

Get the blobs field of the given VM.

_Signature:_

```
(string -> blob ref) map get_blobs (session ref session_ref, VM ref self)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |reference to the object                 |

_Minimum Role:_ read-only

_Return Type:_ `(string -> blob ref) map`

value of the field

### RPC name: get&#95;blocked&#95;operations

_Overview:_

Get the blocked&#95;operations field of the given VM.

_Signature:_

```
(vm_operations -> string) map get_blocked_operations (session ref session_ref, VM ref self)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |reference to the object                 |

_Minimum Role:_ read-only

_Return Type:_ `(vm_operations -> string) map`

value of the field

### RPC name: get&#95;boot&#95;record

**This message is deprecated.**

_Overview:_

Returns a record describing the VM's dynamic state, initialised when the VM boots and updated to reflect runtime configuration changes e.g. CPU hotplug

_Signature:_

```
VM record get_boot_record (session ref session_ref, VM ref self)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |The VM whose boot&#45;time state to return|

_Minimum Role:_ read-only

_Return Type:_ `VM record`

A record describing the VM

### RPC name: get&#95;by&#95;name&#95;label

_Overview:_

Get all the VM instances with the given label.

_Signature:_

```
VM ref set get_by_name_label (session ref session_ref, string label)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`string`                      |`label`                       |label of object to return               |

_Minimum Role:_ read-only

_Return Type:_ `VM ref set`

references to objects with matching names

### RPC name: get&#95;by&#95;uuid

_Overview:_

Get a reference to the VM instance with the specified UUID.

_Signature:_

```
VM ref get_by_uuid (session ref session_ref, string uuid)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`string`                      |`uuid`                        |UUID of object to return                |

_Minimum Role:_ read-only

_Return Type:_ `VM ref`

reference to the object

### RPC name: get&#95;children

_Overview:_

Get the children field of the given VM.

_Signature:_

```
VM ref set get_children (session ref session_ref, VM ref self)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |reference to the object                 |

_Minimum Role:_ read-only

_Return Type:_ `VM ref set`

value of the field

### RPC name: get&#95;consoles

_Overview:_

Get the consoles field of the given VM.

_Signature:_

```
console ref set get_consoles (session ref session_ref, VM ref self)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |reference to the object                 |

_Minimum Role:_ read-only

_Return Type:_ `console ref set`

value of the field

### RPC name: get&#95;cooperative

**This message is deprecated.**

_Overview:_

Return true if the VM is currently 'co&#45;operative' i.e. is expected to reach a balloon target and actually has done

_Signature:_

```
bool get_cooperative (session ref session_ref, VM ref self)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |The VM                                  |

_Minimum Role:_ read-only

_Return Type:_ `bool`

true if the VM is currently 'co&#45;operative'; false otherwise

### RPC name: get&#95;crash&#95;dumps

_Overview:_

Get the crash&#95;dumps field of the given VM.

_Signature:_

```
crashdump ref set get_crash_dumps (session ref session_ref, VM ref self)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |reference to the object                 |

_Minimum Role:_ read-only

_Return Type:_ `crashdump ref set`

value of the field

### RPC name: get&#95;current&#95;operations

_Overview:_

Get the current&#95;operations field of the given VM.

_Signature:_

```
(string -> vm_operations) map get_current_operations (session ref session_ref, VM ref self)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |reference to the object                 |

_Minimum Role:_ read-only

_Return Type:_ `(string -> vm_operations) map`

value of the field

### RPC name: get&#95;data&#95;sources

_Overview:_

_Signature:_

```
data_source record set get_data_sources (session ref session_ref, VM ref self)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |The VM to interrogate                   |

_Minimum Role:_ read-only

_Return Type:_ `data_source record set`

A set of data sources

### RPC name: get&#95;domain&#95;type

_Overview:_

Get the domain&#95;type field of the given VM.

_Signature:_

```
domain_type get_domain_type (session ref session_ref, VM ref self)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |reference to the object                 |

_Minimum Role:_ read-only

_Return Type:_ `domain_type`

value of the field

### RPC name: get&#95;domarch

_Overview:_

Get the domarch field of the given VM.

_Signature:_

```
string get_domarch (session ref session_ref, VM ref self)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |reference to the object                 |

_Minimum Role:_ read-only

_Return Type:_ `string`

value of the field

### RPC name: get&#95;domid

_Overview:_

Get the domid field of the given VM.

_Signature:_

```
int get_domid (session ref session_ref, VM ref self)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |reference to the object                 |

_Minimum Role:_ read-only

_Return Type:_ `int`

value of the field

### RPC name: get&#95;generation&#95;id

_Overview:_

Get the generation&#95;id field of the given VM.

_Signature:_

```
string get_generation_id (session ref session_ref, VM ref self)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |reference to the object                 |

_Minimum Role:_ read-only

_Return Type:_ `string`

value of the field

### RPC name: get&#95;groups

_Overview:_

Get the groups field of the given VM.

_Signature:_

```
VM_group ref set get_groups (session ref session_ref, VM ref self)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |reference to the object                 |

_Minimum Role:_ read-only

_Return Type:_ `VM_group ref set`

value of the field

### RPC name: get&#95;guest&#95;metrics

_Overview:_

Get the guest&#95;metrics field of the given VM.

_Signature:_

```
VM_guest_metrics ref get_guest_metrics (session ref session_ref, VM ref self)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |reference to the object                 |

_Minimum Role:_ read-only

_Return Type:_ `VM_guest_metrics ref`

value of the field

### RPC name: get&#95;ha&#95;always&#95;run

**This message is deprecated.**

_Overview:_

Get the ha&#95;always&#95;run field of the given VM.

_Signature:_

```
bool get_ha_always_run (session ref session_ref, VM ref self)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |reference to the object                 |

_Minimum Role:_ read-only

_Return Type:_ `bool`

value of the field

### RPC name: get&#95;ha&#95;restart&#95;priority

_Overview:_

Get the ha&#95;restart&#95;priority field of the given VM.

_Signature:_

```
string get_ha_restart_priority (session ref session_ref, VM ref self)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |reference to the object                 |

_Minimum Role:_ read-only

_Return Type:_ `string`

value of the field

### RPC name: get&#95;hardware&#95;platform&#95;version

_Overview:_

Get the hardware&#95;platform&#95;version field of the given VM.

_Signature:_

```
int get_hardware_platform_version (session ref session_ref, VM ref self)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |reference to the object                 |

_Minimum Role:_ read-only

_Return Type:_ `int`

value of the field

### RPC name: get&#95;has&#95;vendor&#95;device

_Overview:_

Get the has&#95;vendor&#95;device field of the given VM.

_Signature:_

```
bool get_has_vendor_device (session ref session_ref, VM ref self)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |reference to the object                 |

_Minimum Role:_ read-only

_Return Type:_ `bool`

value of the field

### RPC name: get&#95;HVM&#95;boot&#95;params

_Overview:_

Get the HVM/boot&#95;params field of the given VM.

_Signature:_

```
(string -> string) map get_HVM_boot_params (session ref session_ref, VM ref self)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |reference to the object                 |

_Minimum Role:_ read-only

_Return Type:_ `(string -> string) map`

value of the field

### RPC name: get&#95;HVM&#95;boot&#95;policy

**This message is deprecated.**

_Overview:_

Get the HVM/boot&#95;policy field of the given VM.

_Signature:_

```
string get_HVM_boot_policy (session ref session_ref, VM ref self)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |reference to the object                 |

_Minimum Role:_ read-only

_Return Type:_ `string`

value of the field

### RPC name: get&#95;HVM&#95;shadow&#95;multiplier

_Overview:_

Get the HVM/shadow&#95;multiplier field of the given VM.

_Signature:_

```
float get_HVM_shadow_multiplier (session ref session_ref, VM ref self)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |reference to the object                 |

_Minimum Role:_ read-only

_Return Type:_ `float`

value of the field

### RPC name: get&#95;is&#95;a&#95;snapshot

_Overview:_

Get the is&#95;a&#95;snapshot field of the given VM.

_Signature:_

```
bool get_is_a_snapshot (session ref session_ref, VM ref self)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |reference to the object                 |

_Minimum Role:_ read-only

_Return Type:_ `bool`

value of the field

### RPC name: get&#95;is&#95;a&#95;template

_Overview:_

Get the is&#95;a&#95;template field of the given VM.

_Signature:_

```
bool get_is_a_template (session ref session_ref, VM ref self)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |reference to the object                 |

_Minimum Role:_ read-only

_Return Type:_ `bool`

value of the field

### RPC name: get&#95;is&#95;control&#95;domain

_Overview:_

Get the is&#95;control&#95;domain field of the given VM.

_Signature:_

```
bool get_is_control_domain (session ref session_ref, VM ref self)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |reference to the object                 |

_Minimum Role:_ read-only

_Return Type:_ `bool`

value of the field

### RPC name: get&#95;is&#95;default&#95;template

_Overview:_

Get the is&#95;default&#95;template field of the given VM.

_Signature:_

```
bool get_is_default_template (session ref session_ref, VM ref self)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |reference to the object                 |

_Minimum Role:_ read-only

_Return Type:_ `bool`

value of the field

### RPC name: get&#95;is&#95;snapshot&#95;from&#95;vmpp

**This message is removed.**

_Overview:_

Get the is&#95;snapshot&#95;from&#95;vmpp field of the given VM.

_Signature:_

```
bool get_is_snapshot_from_vmpp (session ref session_ref, VM ref self)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |reference to the object                 |

_Minimum Role:_ read-only

_Return Type:_ `bool`

value of the field

### RPC name: get&#95;is&#95;vmss&#95;snapshot

_Overview:_

Get the is&#95;vmss&#95;snapshot field of the given VM.

_Signature:_

```
bool get_is_vmss_snapshot (session ref session_ref, VM ref self)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |reference to the object                 |

_Minimum Role:_ read-only

_Return Type:_ `bool`

value of the field

### RPC name: get&#95;last&#95;boot&#95;CPU&#95;flags

_Overview:_

Get the last&#95;boot&#95;CPU&#95;flags field of the given VM.

_Signature:_

```
(string -> string) map get_last_boot_CPU_flags (session ref session_ref, VM ref self)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |reference to the object                 |

_Minimum Role:_ read-only

_Return Type:_ `(string -> string) map`

value of the field

### RPC name: get&#95;last&#95;booted&#95;record

_Overview:_

Get the last&#95;booted&#95;record field of the given VM.

_Signature:_

```
string get_last_booted_record (session ref session_ref, VM ref self)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |reference to the object                 |

_Minimum Role:_ read-only

_Return Type:_ `string`

value of the field

### RPC name: get&#95;memory&#95;dynamic&#95;max

_Overview:_

Get the memory/dynamic&#95;max field of the given VM.

_Signature:_

```
int get_memory_dynamic_max (session ref session_ref, VM ref self)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |reference to the object                 |

_Minimum Role:_ read-only

_Return Type:_ `int`

value of the field

### RPC name: get&#95;memory&#95;dynamic&#95;min

_Overview:_

Get the memory/dynamic&#95;min field of the given VM.

_Signature:_

```
int get_memory_dynamic_min (session ref session_ref, VM ref self)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |reference to the object                 |

_Minimum Role:_ read-only

_Return Type:_ `int`

value of the field

### RPC name: get&#95;memory&#95;overhead

_Overview:_

Get the memory/overhead field of the given VM.

_Signature:_

```
int get_memory_overhead (session ref session_ref, VM ref self)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |reference to the object                 |

_Minimum Role:_ read-only

_Return Type:_ `int`

value of the field

### RPC name: get&#95;memory&#95;static&#95;max

_Overview:_

Get the memory/static&#95;max field of the given VM.

_Signature:_

```
int get_memory_static_max (session ref session_ref, VM ref self)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |reference to the object                 |

_Minimum Role:_ read-only

_Return Type:_ `int`

value of the field

### RPC name: get&#95;memory&#95;static&#95;min

_Overview:_

Get the memory/static&#95;min field of the given VM.

_Signature:_

```
int get_memory_static_min (session ref session_ref, VM ref self)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |reference to the object                 |

_Minimum Role:_ read-only

_Return Type:_ `int`

value of the field

### RPC name: get&#95;memory&#95;target

**This message is deprecated.**

_Overview:_

Get the memory/target field of the given VM.

_Signature:_

```
int get_memory_target (session ref session_ref, VM ref self)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |reference to the object                 |

_Minimum Role:_ read-only

_Return Type:_ `int`

value of the field

### RPC name: get&#95;metrics

_Overview:_

Get the metrics field of the given VM.

_Signature:_

```
VM_metrics ref get_metrics (session ref session_ref, VM ref self)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |reference to the object                 |

_Minimum Role:_ read-only

_Return Type:_ `VM_metrics ref`

value of the field

### RPC name: get&#95;name&#95;description

_Overview:_

Get the name/description field of the given VM.

_Signature:_

```
string get_name_description (session ref session_ref, VM ref self)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |reference to the object                 |

_Minimum Role:_ read-only

_Return Type:_ `string`

value of the field

### RPC name: get&#95;name&#95;label

_Overview:_

Get the name/label field of the given VM.

_Signature:_

```
string get_name_label (session ref session_ref, VM ref self)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |reference to the object                 |

_Minimum Role:_ read-only

_Return Type:_ `string`

value of the field

### RPC name: get&#95;NVRAM

_Overview:_

Get the NVRAM field of the given VM.

_Signature:_

```
(string -> string) map get_NVRAM (session ref session_ref, VM ref self)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |reference to the object                 |

_Minimum Role:_ read-only

_Return Type:_ `(string -> string) map`

value of the field

### RPC name: get&#95;order

_Overview:_

Get the order field of the given VM.

_Signature:_

```
int get_order (session ref session_ref, VM ref self)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |reference to the object                 |

_Minimum Role:_ read-only

_Return Type:_ `int`

value of the field

### RPC name: get&#95;other&#95;config

_Overview:_

Get the other&#95;config field of the given VM.

_Signature:_

```
(string -> string) map get_other_config (session ref session_ref, VM ref self)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |reference to the object                 |

_Minimum Role:_ read-only

_Return Type:_ `(string -> string) map`

value of the field

### RPC name: get&#95;parent

_Overview:_

Get the parent field of the given VM.

_Signature:_

```
VM ref get_parent (session ref session_ref, VM ref self)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |reference to the object                 |

_Minimum Role:_ read-only

_Return Type:_ `VM ref`

value of the field

### RPC name: get&#95;PCI&#95;bus

**This message is deprecated.**

_Overview:_

Get the PCI&#95;bus field of the given VM.

_Signature:_

```
string get_PCI_bus (session ref session_ref, VM ref self)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |reference to the object                 |

_Minimum Role:_ read-only

_Return Type:_ `string`

value of the field

### RPC name: get&#95;pending&#95;guidances

_Overview:_

Get the pending&#95;guidances field of the given VM.

_Signature:_

```
update_guidances set get_pending_guidances (session ref session_ref, VM ref self)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |reference to the object                 |

_Minimum Role:_ read-only

_Return Type:_ `update_guidances set`

value of the field

### RPC name: get&#95;pending&#95;guidances&#95;full

_Overview:_

Get the pending&#95;guidances&#95;full field of the given VM.

_Signature:_

```
update_guidances set get_pending_guidances_full (session ref session_ref, VM ref self)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |reference to the object                 |

_Minimum Role:_ read-only

_Return Type:_ `update_guidances set`

value of the field

### RPC name: get&#95;pending&#95;guidances&#95;recommended

_Overview:_

Get the pending&#95;guidances&#95;recommended field of the given VM.

_Signature:_

```
update_guidances set get_pending_guidances_recommended (session ref session_ref, VM ref self)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |reference to the object                 |

_Minimum Role:_ read-only

_Return Type:_ `update_guidances set`

value of the field

### RPC name: get&#95;platform

_Overview:_

Get the platform field of the given VM.

_Signature:_

```
(string -> string) map get_platform (session ref session_ref, VM ref self)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |reference to the object                 |

_Minimum Role:_ read-only

_Return Type:_ `(string -> string) map`

value of the field

### RPC name: get&#95;possible&#95;hosts

_Overview:_

Return the list of hosts on which this VM may run.

_Signature:_

```
host ref set get_possible_hosts (session ref session_ref, VM ref vm)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`vm`                          |The VM                                  |

_Minimum Role:_ read-only

_Return Type:_ `host ref set`

The possible hosts

### RPC name: get&#95;power&#95;state

_Overview:_

Get the power&#95;state field of the given VM.

_Signature:_

```
vm_power_state get_power_state (session ref session_ref, VM ref self)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |reference to the object                 |

_Minimum Role:_ read-only

_Return Type:_ `vm_power_state`

value of the field

### RPC name: get&#95;protection&#95;policy

**This message is deprecated.**

_Overview:_

Get the protection&#95;policy field of the given VM.

_Signature:_

```
VMPP ref get_protection_policy (session ref session_ref, VM ref self)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |reference to the object                 |

_Minimum Role:_ read-only

_Return Type:_ `VMPP ref`

value of the field

### RPC name: get&#95;PV&#95;args

_Overview:_

Get the PV/args field of the given VM.

_Signature:_

```
string get_PV_args (session ref session_ref, VM ref self)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |reference to the object                 |

_Minimum Role:_ read-only

_Return Type:_ `string`

value of the field

### RPC name: get&#95;PV&#95;bootloader

_Overview:_

Get the PV/bootloader field of the given VM.

_Signature:_

```
string get_PV_bootloader (session ref session_ref, VM ref self)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |reference to the object                 |

_Minimum Role:_ read-only

_Return Type:_ `string`

value of the field

### RPC name: get&#95;PV&#95;bootloader&#95;args

_Overview:_

Get the PV/bootloader&#95;args field of the given VM.

_Signature:_

```
string get_PV_bootloader_args (session ref session_ref, VM ref self)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |reference to the object                 |

_Minimum Role:_ read-only

_Return Type:_ `string`

value of the field

### RPC name: get&#95;PV&#95;kernel

_Overview:_

Get the PV/kernel field of the given VM.

_Signature:_

```
string get_PV_kernel (session ref session_ref, VM ref self)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |reference to the object                 |

_Minimum Role:_ read-only

_Return Type:_ `string`

value of the field

### RPC name: get&#95;PV&#95;legacy&#95;args

_Overview:_

Get the PV/legacy&#95;args field of the given VM.

_Signature:_

```
string get_PV_legacy_args (session ref session_ref, VM ref self)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |reference to the object                 |

_Minimum Role:_ read-only

_Return Type:_ `string`

value of the field

### RPC name: get&#95;PV&#95;ramdisk

_Overview:_

Get the PV/ramdisk field of the given VM.

_Signature:_

```
string get_PV_ramdisk (session ref session_ref, VM ref self)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |reference to the object                 |

_Minimum Role:_ read-only

_Return Type:_ `string`

value of the field

### RPC name: get&#95;recommendations

_Overview:_

Get the recommendations field of the given VM.

_Signature:_

```
string get_recommendations (session ref session_ref, VM ref self)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |reference to the object                 |

_Minimum Role:_ read-only

_Return Type:_ `string`

value of the field

### RPC name: get&#95;record

_Overview:_

Get a record containing the current state of the given VM.

_Signature:_

```
VM record get_record (session ref session_ref, VM ref self)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |reference to the object                 |

_Minimum Role:_ read-only

_Return Type:_ `VM record`

all fields from the object

### RPC name: get&#95;reference&#95;label

_Overview:_

Get the reference&#95;label field of the given VM.

_Signature:_

```
string get_reference_label (session ref session_ref, VM ref self)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |reference to the object                 |

_Minimum Role:_ read-only

_Return Type:_ `string`

value of the field

### RPC name: get&#95;requires&#95;reboot

_Overview:_

Get the requires&#95;reboot field of the given VM.

_Signature:_

```
bool get_requires_reboot (session ref session_ref, VM ref self)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |reference to the object                 |

_Minimum Role:_ read-only

_Return Type:_ `bool`

value of the field

### RPC name: get&#95;resident&#95;on

_Overview:_

Get the resident&#95;on field of the given VM.

_Signature:_

```
host ref get_resident_on (session ref session_ref, VM ref self)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |reference to the object                 |

_Minimum Role:_ read-only

_Return Type:_ `host ref`

value of the field

### RPC name: get&#95;scheduled&#95;to&#95;be&#95;resident&#95;on

_Overview:_

Get the scheduled&#95;to&#95;be&#95;resident&#95;on field of the given VM.

_Signature:_

```
host ref get_scheduled_to_be_resident_on (session ref session_ref, VM ref self)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |reference to the object                 |

_Minimum Role:_ read-only

_Return Type:_ `host ref`

value of the field

### RPC name: get&#95;secureboot&#95;readiness

_Overview:_

Return the secureboot readiness of the VM

_Signature:_

```
vm_secureboot_readiness get_secureboot_readiness (session ref session_ref, VM ref self)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |The VM                                  |

_Minimum Role:_ pool-admin

_Return Type:_ `vm_secureboot_readiness`

The secureboot readiness of the VM

### RPC name: get&#95;shutdown&#95;delay

_Overview:_

Get the shutdown&#95;delay field of the given VM.

_Signature:_

```
int get_shutdown_delay (session ref session_ref, VM ref self)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |reference to the object                 |

_Minimum Role:_ read-only

_Return Type:_ `int`

value of the field

### RPC name: get&#95;snapshot&#95;info

_Overview:_

Get the snapshot&#95;info field of the given VM.

_Signature:_

```
(string -> string) map get_snapshot_info (session ref session_ref, VM ref self)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |reference to the object                 |

_Minimum Role:_ read-only

_Return Type:_ `(string -> string) map`

value of the field

### RPC name: get&#95;snapshot&#95;metadata

_Overview:_

Get the snapshot&#95;metadata field of the given VM.

_Signature:_

```
string get_snapshot_metadata (session ref session_ref, VM ref self)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |reference to the object                 |

_Minimum Role:_ read-only

_Return Type:_ `string`

value of the field

### RPC name: get&#95;snapshot&#95;of

_Overview:_

Get the snapshot&#95;of field of the given VM.

_Signature:_

```
VM ref get_snapshot_of (session ref session_ref, VM ref self)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |reference to the object                 |

_Minimum Role:_ read-only

_Return Type:_ `VM ref`

value of the field

### RPC name: get&#95;snapshot&#95;schedule

_Overview:_

Get the snapshot&#95;schedule field of the given VM.

_Signature:_

```
VMSS ref get_snapshot_schedule (session ref session_ref, VM ref self)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |reference to the object                 |

_Minimum Role:_ read-only

_Return Type:_ `VMSS ref`

value of the field

### RPC name: get&#95;snapshot&#95;time

_Overview:_

Get the snapshot&#95;time field of the given VM.

_Signature:_

```
datetime get_snapshot_time (session ref session_ref, VM ref self)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |reference to the object                 |

_Minimum Role:_ read-only

_Return Type:_ `datetime`

value of the field

### RPC name: get&#95;snapshots

_Overview:_

Get the snapshots field of the given VM.

_Signature:_

```
VM ref set get_snapshots (session ref session_ref, VM ref self)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |reference to the object                 |

_Minimum Role:_ read-only

_Return Type:_ `VM ref set`

value of the field

### RPC name: get&#95;SRs&#95;required&#95;for&#95;recovery

_Overview:_

List all the SR's that are required for the VM to be recovered

_Signature:_

```
SR ref set get_SRs_required_for_recovery (session ref session_ref, VM ref self, session ref session_to)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |The VM for which the SRs have to be recovered|
|`session ref`                 |`session_to`                  |The session to which the SRs of the VM have to be recovered.|

_Minimum Role:_ read-only

_Return Type:_ `SR ref set`

refs for SRs required to recover the VM

### RPC name: get&#95;start&#95;delay

_Overview:_

Get the start&#95;delay field of the given VM.

_Signature:_

```
int get_start_delay (session ref session_ref, VM ref self)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |reference to the object                 |

_Minimum Role:_ read-only

_Return Type:_ `int`

value of the field

### RPC name: get&#95;suspend&#95;SR

_Overview:_

Get the suspend&#95;SR field of the given VM.

_Signature:_

```
SR ref get_suspend_SR (session ref session_ref, VM ref self)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |reference to the object                 |

_Minimum Role:_ read-only

_Return Type:_ `SR ref`

value of the field

### RPC name: get&#95;suspend&#95;VDI

_Overview:_

Get the suspend&#95;VDI field of the given VM.

_Signature:_

```
VDI ref get_suspend_VDI (session ref session_ref, VM ref self)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |reference to the object                 |

_Minimum Role:_ read-only

_Return Type:_ `VDI ref`

value of the field

### RPC name: get&#95;tags

_Overview:_

Get the tags field of the given VM.

_Signature:_

```
string set get_tags (session ref session_ref, VM ref self)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |reference to the object                 |

_Minimum Role:_ read-only

_Return Type:_ `string set`

value of the field

### RPC name: get&#95;transportable&#95;snapshot&#95;id

_Overview:_

Get the transportable&#95;snapshot&#95;id field of the given VM.

_Signature:_

```
string get_transportable_snapshot_id (session ref session_ref, VM ref self)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |reference to the object                 |

_Minimum Role:_ read-only

_Return Type:_ `string`

value of the field

### RPC name: get&#95;user&#95;version

_Overview:_

Get the user&#95;version field of the given VM.

_Signature:_

```
int get_user_version (session ref session_ref, VM ref self)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |reference to the object                 |

_Minimum Role:_ read-only

_Return Type:_ `int`

value of the field

### RPC name: get&#95;uuid

_Overview:_

Get the uuid field of the given VM.

_Signature:_

```
string get_uuid (session ref session_ref, VM ref self)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |reference to the object                 |

_Minimum Role:_ read-only

_Return Type:_ `string`

value of the field

### RPC name: get&#95;VBDs

_Overview:_

Get the VBDs field of the given VM.

_Signature:_

```
VBD ref set get_VBDs (session ref session_ref, VM ref self)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |reference to the object                 |

_Minimum Role:_ read-only

_Return Type:_ `VBD ref set`

value of the field

### RPC name: get&#95;VCPUs&#95;at&#95;startup

_Overview:_

Get the VCPUs/at&#95;startup field of the given VM.

_Signature:_

```
int get_VCPUs_at_startup (session ref session_ref, VM ref self)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |reference to the object                 |

_Minimum Role:_ read-only

_Return Type:_ `int`

value of the field

### RPC name: get&#95;VCPUs&#95;max

_Overview:_

Get the VCPUs/max field of the given VM.

_Signature:_

```
int get_VCPUs_max (session ref session_ref, VM ref self)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |reference to the object                 |

_Minimum Role:_ read-only

_Return Type:_ `int`

value of the field

### RPC name: get&#95;VCPUs&#95;params

_Overview:_

Get the VCPUs/params field of the given VM.

_Signature:_

```
(string -> string) map get_VCPUs_params (session ref session_ref, VM ref self)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |reference to the object                 |

_Minimum Role:_ read-only

_Return Type:_ `(string -> string) map`

value of the field

### RPC name: get&#95;version

_Overview:_

Get the version field of the given VM.

_Signature:_

```
int get_version (session ref session_ref, VM ref self)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |reference to the object                 |

_Minimum Role:_ read-only

_Return Type:_ `int`

value of the field

### RPC name: get&#95;VGPUs

_Overview:_

Get the VGPUs field of the given VM.

_Signature:_

```
VGPU ref set get_VGPUs (session ref session_ref, VM ref self)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |reference to the object                 |

_Minimum Role:_ read-only

_Return Type:_ `VGPU ref set`

value of the field

### RPC name: get&#95;VIFs

_Overview:_

Get the VIFs field of the given VM.

_Signature:_

```
VIF ref set get_VIFs (session ref session_ref, VM ref self)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |reference to the object                 |

_Minimum Role:_ read-only

_Return Type:_ `VIF ref set`

value of the field

### RPC name: get&#95;VTPMs

_Overview:_

Get the VTPMs field of the given VM.

_Signature:_

```
VTPM ref set get_VTPMs (session ref session_ref, VM ref self)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |reference to the object                 |

_Minimum Role:_ read-only

_Return Type:_ `VTPM ref set`

value of the field

### RPC name: get&#95;VUSBs

_Overview:_

Get the VUSBs field of the given VM.

_Signature:_

```
VUSB ref set get_VUSBs (session ref session_ref, VM ref self)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |reference to the object                 |

_Minimum Role:_ read-only

_Return Type:_ `VUSB ref set`

value of the field

### RPC name: get&#95;xenstore&#95;data

_Overview:_

Get the xenstore&#95;data field of the given VM.

_Signature:_

```
(string -> string) map get_xenstore_data (session ref session_ref, VM ref self)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |reference to the object                 |

_Minimum Role:_ read-only

_Return Type:_ `(string -> string) map`

value of the field

### RPC name: hard&#95;reboot

_Overview:_

Stop executing the specified VM without attempting a clean shutdown and immediately restart the VM.

_Signature:_

```
void hard_reboot (session ref session_ref, VM ref vm)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`vm`                          |The VM to reboot                        |

_Minimum Role:_ vm-operator

_Return Type:_ `void`

_Possible Error Codes:_ `VM_BAD_POWER_STATE`, `OTHER_OPERATION_IN_PROGRESS`, `OPERATION_NOT_ALLOWED`, `VM_IS_TEMPLATE`

### RPC name: hard&#95;shutdown

_Overview:_

Stop executing the specified VM without attempting a clean shutdown.

_Signature:_

```
void hard_shutdown (session ref session_ref, VM ref vm)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`vm`                          |The VM to destroy                       |

_Minimum Role:_ vm-operator

_Return Type:_ `void`

_Possible Error Codes:_ `VM_BAD_POWER_STATE`, `OTHER_OPERATION_IN_PROGRESS`, `OPERATION_NOT_ALLOWED`, `VM_IS_TEMPLATE`

### RPC name: import

_Overview:_

Import an XVA from a URI

_Signature:_

```
VM ref set import (session ref session_ref, string url, SR ref sr, bool full_restore, bool force)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`string`                      |`url`                         |The URL of the XVA file                 |
|`SR ref`                      |`sr`                          |The destination SR for the disks        |
|`bool`                        |`full_restore`                |Perform a full restore                  |
|`bool`                        |`force`                       |Force the import                        |

_Minimum Role:_ pool-operator

_Return Type:_ `VM ref set`

Imported VM reference

### RPC name: import&#95;convert

_Overview:_

Import using a conversion service.

_Signature:_

```
void import_convert (session ref session_ref, string type, string username, string password, SR ref sr, (string -> string) map remote_config)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`string`                      |`type`                        |Type of the conversion                  |
|`string`                      |`username`                    |Admin username on the host              |
|`string`                      |`password`                    |Password on the host                    |
|`SR ref`                      |`sr`                          |The destination SR                      |
|`(string -> string) map`      |`remote_config`               |Remote configuration options            |

_Minimum Role:_ vm-admin

_Return Type:_ `void`

### RPC name: maximise&#95;memory

_Overview:_

Returns the maximum amount of guest memory which will fit, together with overheads, in the supplied amount of physical memory. If 'exact' is true then an exact calculation is performed using the VM's current settings. If 'exact' is false then a more conservative approximation is used

_Signature:_

```
int maximise_memory (session ref session_ref, VM ref self, int total, bool approximate)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |The VM                                  |
|`int`                         |`total`                       |Total amount of physical RAM to fit within|
|`bool`                        |`approximate`                 |If false the limit is calculated with the guest's current exact configuration. Otherwise a more approximate calculation is performed|

_Minimum Role:_ read-only

_Return Type:_ `int`

The maximum possible static&#45;max

### RPC name: migrate&#95;send

_Overview:_

Migrate the VM to another host.  This can only be called when the specified VM is in the Running state.

_Signature:_

```
VM ref migrate_send (session ref session_ref, VM ref vm, (string -> string) map dest, bool live, (VDI ref -> SR ref) map vdi_map, (VIF ref -> network ref) map vif_map, (string -> string) map options, (VGPU ref -> GPU_group ref) map vgpu_map)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`vm`                          |The VM                                  |
|`(string -> string) map`      |`dest`                        |The result of a Host.migrate&#95;receive call.|
|`bool`                        |`live`                        |Live migration                          |
|`(VDI ref -> SR ref) map`     |`vdi_map`                     |Map of source VDI to destination SR     |
|`(VIF ref -> network ref) map`|`vif_map`                     |Map of source VIF to destination network|
|`(string -> string) map`      |`options`                     |Other parameters                        |
|`(VGPU ref -> GPU_group ref) map`|`vgpu_map`                    |Map of source vGPU to destination GPU group|

_Minimum Role:_ vm-power-admin

_Return Type:_ `VM ref`

The reference of the newly created VM in the destination pool

_Possible Error Codes:_ `VM_BAD_POWER_STATE`, `LICENCE_RESTRICTION`

### RPC name: pause

_Overview:_

Pause the specified VM. This can only be called when the specified VM is in the Running state.

_Signature:_

```
void pause (session ref session_ref, VM ref vm)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`vm`                          |The VM to pause                         |

_Minimum Role:_ vm-operator

_Return Type:_ `void`

_Possible Error Codes:_ `VM_BAD_POWER_STATE`, `OTHER_OPERATION_IN_PROGRESS`, `OPERATION_NOT_ALLOWED`, `VM_IS_TEMPLATE`

### RPC name: pool&#95;migrate

_Overview:_

Migrate a VM to another Host.

_Signature:_

```
void pool_migrate (session ref session_ref, VM ref vm, host ref host, (string -> string) map options)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`vm`                          |The VM to migrate                       |
|`host ref`                    |`host`                        |The target host                         |
|`(string -> string) map`      |`options`                     |Extra configuration operations: force, live, copy, compress. Each is a boolean option, taking 'true' or 'false' as a value. Option 'compress' controls the use of stream compression during migration.|

_Minimum Role:_ client-cert

_Return Type:_ `void`

_Possible Error Codes:_ `VM_BAD_POWER_STATE`, `OTHER_OPERATION_IN_PROGRESS`, `VM_IS_TEMPLATE`, `OPERATION_NOT_ALLOWED`, `VM_BAD_POWER_STATE`

### RPC name: power&#95;state&#95;reset

_Overview:_

Reset the power&#45;state of the VM to halted in the database only. &#40;Used to recover from slave failures in pooling scenarios by resetting the power&#45;states of VMs running on dead slaves to halted.&#41; This is a potentially dangerous operation; use with care.

_Signature:_

```
void power_state_reset (session ref session_ref, VM ref vm)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`vm`                          |The VM to reset                         |

_Minimum Role:_ pool-operator

_Return Type:_ `void`

### RPC name: provision

_Overview:_

Inspects the disk configuration contained within the VM's other&#95;config, creates VDIs and VBDs and then executes any applicable post&#45;install script.

_Signature:_

```
void provision (session ref session_ref, VM ref vm)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`vm`                          |The VM to be provisioned                |

_Minimum Role:_ vm-admin

_Return Type:_ `void`

_Possible Error Codes:_ `VM_BAD_POWER_STATE`, `SR_FULL`, `OPERATION_NOT_ALLOWED`, `LICENCE_RESTRICTION`

### RPC name: query&#95;data&#95;source

_Overview:_

Query the latest value of the specified data source

_Signature:_

```
float query_data_source (session ref session_ref, VM ref self, string data_source)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |The VM                                  |
|`string`                      |`data_source`                 |The data source to query                |

_Minimum Role:_ read-only

_Return Type:_ `float`

The latest value, averaged over the last 5 seconds

### RPC name: query&#95;services

_Overview:_

Query the system services advertised by this VM and register them. This can only be applied to a system domain.

_Signature:_

```
(string -> string) map query_services (session ref session_ref, VM ref self)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |The VM                                  |

_Minimum Role:_ pool-admin

_Return Type:_ `(string -> string) map`

map of service type to name

### RPC name: record&#95;data&#95;source

_Overview:_

Start recording the specified data source

_Signature:_

```
void record_data_source (session ref session_ref, VM ref self, string data_source)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |The VM                                  |
|`string`                      |`data_source`                 |The data source to record               |

_Minimum Role:_ vm-admin

_Return Type:_ `void`

### RPC name: recover

_Overview:_

Recover the VM

_Signature:_

```
void recover (session ref session_ref, VM ref self, session ref session_to, bool force)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |The VM to recover                       |
|`session ref`                 |`session_to`                  |The session to which the VM is to be recovered.|
|`bool`                        |`force`                       |Whether the VM should replace newer versions of itself.|

_Minimum Role:_ read-only

_Return Type:_ `void`

### RPC name: remove&#95;from&#95;blocked&#95;operations

_Overview:_

Update list of operations which have been explicitly blocked and an error code

_Signature:_

```
void remove_from_blocked_operations (session ref session_ref, VM ref self, vm_operations key)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |The VM                                  |
|`vm_operations`               |`key`                         |Blocked operation                       |

_Minimum Role:_ vm-admin

_Return Type:_ `void`

### RPC name: remove&#95;from&#95;HVM&#95;boot&#95;params

_Overview:_

Remove the given key and its corresponding value from the HVM/boot&#95;params field of the given VM.  If the key is not in that Map, then do nothing.

_Signature:_

```
void remove_from_HVM_boot_params (session ref session_ref, VM ref self, string key)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |reference to the object                 |
|`string`                      |`key`                         |Key to remove                           |

_Minimum Role:_ vm-admin

_Return Type:_ `void`

### RPC name: remove&#95;from&#95;NVRAM

_Overview:_

_Signature:_

```
void remove_from_NVRAM (session ref session_ref, VM ref self, string key)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |The VM                                  |
|`string`                      |`key`                         |The key                                 |

_Minimum Role:_ vm-admin

_Return Type:_ `void`

### RPC name: remove&#95;from&#95;other&#95;config

_Overview:_

Remove the given key and its corresponding value from the other&#95;config field of the given VM.  If the key is not in that Map, then do nothing.

_Signature:_

```
void remove_from_other_config (session ref session_ref, VM ref self, string key)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |reference to the object                 |
|`string`                      |`key`                         |Key to remove                           |

_Minimum Role:_ vm-admin

_Return Type:_ `void`

### RPC name: remove&#95;from&#95;platform

_Overview:_

Remove the given key and its corresponding value from the platform field of the given VM.  If the key is not in that Map, then do nothing.

_Signature:_

```
void remove_from_platform (session ref session_ref, VM ref self, string key)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |reference to the object                 |
|`string`                      |`key`                         |Key to remove                           |

_Minimum Role:_ vm-admin

_Return Type:_ `void`

### RPC name: remove&#95;from&#95;VCPUs&#95;params

_Overview:_

Remove the given key and its corresponding value from the VCPUs/params field of the given VM.  If the key is not in that Map, then do nothing.

_Signature:_

```
void remove_from_VCPUs_params (session ref session_ref, VM ref self, string key)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |reference to the object                 |
|`string`                      |`key`                         |Key to remove                           |

_Minimum Role:_ vm-admin

_Return Type:_ `void`

### RPC name: remove&#95;from&#95;xenstore&#95;data

_Overview:_

Remove the given key and its corresponding value from the xenstore&#95;data field of the given VM.  If the key is not in that Map, then do nothing.

_Signature:_

```
void remove_from_xenstore_data (session ref session_ref, VM ref self, string key)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |reference to the object                 |
|`string`                      |`key`                         |Key to remove                           |

_Minimum Role:_ vm-admin

_Return Type:_ `void`

### RPC name: remove&#95;tags

_Overview:_

Remove the given value from the tags field of the given VM.  If the value is not in that Set, then do nothing.

_Signature:_

```
void remove_tags (session ref session_ref, VM ref self, string value)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |reference to the object                 |
|`string`                      |`value`                       |Value to remove                         |

_Minimum Role:_ vm-operator

_Return Type:_ `void`

### RPC name: restart&#95;device&#95;models

_Overview:_

Restart device models of the VM

_Signature:_

```
void restart_device_models (session ref session_ref, VM ref self)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |The VM                                  |

_Minimum Role:_ client-cert

_Return Type:_ `void`

_Possible Error Codes:_ `VM_BAD_POWER_STATE`, `OTHER_OPERATION_IN_PROGRESS`, `VM_IS_TEMPLATE`, `OPERATION_NOT_ALLOWED`, `VM_BAD_POWER_STATE`

### RPC name: resume

_Overview:_

Awaken the specified VM and resume it.  This can only be called when the specified VM is in the Suspended state.

_Signature:_

```
void resume (session ref session_ref, VM ref vm, bool start_paused, bool force)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`vm`                          |The VM to resume                        |
|`bool`                        |`start_paused`                |Resume VM in paused state if set to true.|
|`bool`                        |`force`                       |Attempt to force the VM to resume. If this flag is false then the VM may fail pre&#45;resume safety checks &#40;e.g. if the CPU the VM was running on looks substantially different to the current one&#41;|

_Minimum Role:_ vm-operator

_Return Type:_ `void`

_Possible Error Codes:_ `VM_BAD_POWER_STATE`, `OPERATION_NOT_ALLOWED`, `VM_IS_TEMPLATE`

### RPC name: resume&#95;on

_Overview:_

Awaken the specified VM and resume it on a particular Host.  This can only be called when the specified VM is in the Suspended state.

_Signature:_

```
void resume_on (session ref session_ref, VM ref vm, host ref host, bool start_paused, bool force)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`vm`                          |The VM to resume                        |
|`host ref`                    |`host`                        |The Host on which to resume the VM      |
|`bool`                        |`start_paused`                |Resume VM in paused state if set to true.|
|`bool`                        |`force`                       |Attempt to force the VM to resume. If this flag is false then the VM may fail pre&#45;resume safety checks &#40;e.g. if the CPU the VM was running on looks substantially different to the current one&#41;|

_Minimum Role:_ client-cert

_Return Type:_ `void`

_Possible Error Codes:_ `VM_BAD_POWER_STATE`, `OPERATION_NOT_ALLOWED`, `VM_IS_TEMPLATE`

### RPC name: retrieve&#95;wlb&#95;recommendations

_Overview:_

Returns mapping of hosts to ratings, indicating the suitability of starting the VM at that location according to wlb. Rating is replaced with an error if the VM cannot boot there.

_Signature:_

```
(host ref -> string set) map retrieve_wlb_recommendations (session ref session_ref, VM ref vm)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`vm`                          |The VM                                  |

_Minimum Role:_ read-only

_Return Type:_ `(host ref -> string set) map`

The potential hosts and their corresponding recommendations or errors

### RPC name: revert

_Overview:_

Reverts the specified VM to a previous state.

_Signature:_

```
void revert (session ref session_ref, VM ref snapshot)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`snapshot`                    |The snapshotted state that we revert to |

_Minimum Role:_ vm-power-admin

_Return Type:_ `void`

_Possible Error Codes:_ `VM_BAD_POWER_STATE`, `OPERATION_NOT_ALLOWED`, `SR_FULL`, `VM_REVERT_FAILED`

### RPC name: send&#95;sysrq

_Overview:_

Send the given key as a sysrq to this VM.  The key is specified as a single character &#40;a String of length 1&#41;.  This can only be called when the specified VM is in the Running state.

_Signature:_

```
void send_sysrq (session ref session_ref, VM ref vm, string key)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`vm`                          |The VM                                  |
|`string`                      |`key`                         |The key to send                         |

_Minimum Role:_ pool-admin

_Return Type:_ `void`

_Possible Error Codes:_ `VM_BAD_POWER_STATE`

### RPC name: send&#95;trigger

_Overview:_

Send the named trigger to this VM.  This can only be called when the specified VM is in the Running state.

_Signature:_

```
void send_trigger (session ref session_ref, VM ref vm, string trigger)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`vm`                          |The VM                                  |
|`string`                      |`trigger`                     |The trigger to send                     |

_Minimum Role:_ pool-admin

_Return Type:_ `void`

_Possible Error Codes:_ `VM_BAD_POWER_STATE`

### RPC name: set&#95;actions&#95;after&#95;crash

_Overview:_

Sets the actions&#95;after&#95;crash parameter

_Signature:_

```
void set_actions_after_crash (session ref session_ref, VM ref self, on_crash_behaviour value)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |The VM to set                           |
|`on_crash_behaviour`          |`value`                       |The new value to set                    |

_Minimum Role:_ vm-admin

_Return Type:_ `void`

### RPC name: set&#95;actions&#95;after&#95;reboot

_Overview:_

Set the actions/after&#95;reboot field of the given VM.

_Signature:_

```
void set_actions_after_reboot (session ref session_ref, VM ref self, on_normal_exit value)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |reference to the object                 |
|`on_normal_exit`              |`value`                       |New value to set                        |

_Minimum Role:_ vm-admin

_Return Type:_ `void`

### RPC name: set&#95;actions&#95;after&#95;shutdown

_Overview:_

Set the actions/after&#95;shutdown field of the given VM.

_Signature:_

```
void set_actions_after_shutdown (session ref session_ref, VM ref self, on_normal_exit value)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |reference to the object                 |
|`on_normal_exit`              |`value`                       |New value to set                        |

_Minimum Role:_ vm-admin

_Return Type:_ `void`

### RPC name: set&#95;actions&#95;after&#95;softreboot

_Overview:_

Set the actions/after&#95;softreboot field of the given VM.

_Signature:_

```
void set_actions_after_softreboot (session ref session_ref, VM ref self, on_softreboot_behavior value)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |reference to the object                 |
|`on_softreboot_behavior`      |`value`                       |New value to set                        |

_Minimum Role:_ vm-admin

_Return Type:_ `void`

### RPC name: set&#95;affinity

_Overview:_

Set the affinity field of the given VM.

_Signature:_

```
void set_affinity (session ref session_ref, VM ref self, host ref value)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |reference to the object                 |
|`host ref`                    |`value`                       |New value to set                        |

_Minimum Role:_ vm-power-admin

_Return Type:_ `void`

### RPC name: set&#95;appliance

_Overview:_

Assign this VM to an appliance.

_Signature:_

```
void set_appliance (session ref session_ref, VM ref self, VM_appliance ref value)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |The VM to assign to an appliance.       |
|`VM_appliance ref`            |`value`                       |The appliance to which this VM should be assigned.|

_Minimum Role:_ pool-operator

_Return Type:_ `void`

### RPC name: set&#95;bios&#95;strings

_Overview:_

Set custom BIOS strings to this VM. VM will be given a default set of BIOS strings, only some of which can be overridden by the supplied values. Allowed keys are: 'bios&#45;vendor', 'bios&#45;version', 'system&#45;manufacturer', 'system&#45;product&#45;name', 'system&#45;version', 'system&#45;serial&#45;number', 'enclosure&#45;asset&#45;tag', 'baseboard&#45;manufacturer', 'baseboard&#45;product&#45;name', 'baseboard&#45;version', 'baseboard&#45;serial&#45;number', 'baseboard&#45;asset&#45;tag', 'baseboard&#45;location&#45;in&#45;chassis', 'enclosure&#45;asset&#45;tag'

_Signature:_

```
void set_bios_strings (session ref session_ref, VM ref self, (string -> string) map value)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |The VM to modify                        |
|`(string -> string) map`      |`value`                       |The custom BIOS strings as a list of key&#45;value pairs|

_Minimum Role:_ vm-admin

_Return Type:_ `void`

_Possible Error Codes:_ `VM_BIOS_STRINGS_ALREADY_SET`, `INVALID_VALUE`

### RPC name: set&#95;blocked&#95;operations

_Overview:_

Update list of operations which have been explicitly blocked and an error code

_Signature:_

```
void set_blocked_operations (session ref session_ref, VM ref self, (vm_operations -> string) map value)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |The VM                                  |
|`(vm_operations -> string) map`|`value`                       |Blocked operations                      |

_Minimum Role:_ vm-admin

_Return Type:_ `void`

### RPC name: set&#95;domain&#95;type

_Overview:_

Set the VM.domain&#95;type field of the given VM, which will take effect when it is next started

_Signature:_

```
void set_domain_type (session ref session_ref, VM ref self, domain_type value)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |The VM                                  |
|`domain_type`                 |`value`                       |The new domain type                     |

_Minimum Role:_ vm-admin

_Return Type:_ `void`

### RPC name: set&#95;groups

_Overview:_

Associate this VM with VM groups.

_Signature:_

```
void set_groups (session ref session_ref, VM ref self, VM_group ref set value)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |The VM                                  |
|`VM_group ref set`            |`value`                       |The VM groups to set                    |

_Minimum Role:_ vm-admin

_Return Type:_ `void`

_Possible Error Codes:_ `OPERATION_NOT_ALLOWED`

### RPC name: set&#95;ha&#95;always&#95;run

**This message is deprecated.**

_Overview:_

Set the value of the ha&#95;always&#95;run

_Signature:_

```
void set_ha_always_run (session ref session_ref, VM ref self, bool value)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |The VM                                  |
|`bool`                        |`value`                       |The value                               |

_Minimum Role:_ pool-operator

_Return Type:_ `void`

### RPC name: set&#95;ha&#95;restart&#95;priority

_Overview:_

Set the value of the ha&#95;restart&#95;priority field

_Signature:_

```
void set_ha_restart_priority (session ref session_ref, VM ref self, string value)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |The VM                                  |
|`string`                      |`value`                       |The value                               |

_Minimum Role:_ pool-operator

_Return Type:_ `void`

### RPC name: set&#95;hardware&#95;platform&#95;version

_Overview:_

Set the hardware&#95;platform&#95;version field of the given VM.

_Signature:_

```
void set_hardware_platform_version (session ref session_ref, VM ref self, int value)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |reference to the object                 |
|`int`                         |`value`                       |New value to set                        |

_Minimum Role:_ vm-admin

_Return Type:_ `void`

### RPC name: set&#95;has&#95;vendor&#95;device

_Overview:_

Controls whether, when the VM starts in HVM mode, its virtual hardware will include the emulated PCI device for which drivers may be available through Windows Update. Usually this should never be changed on a VM on which Windows has been installed: changing it on such a VM is likely to lead to a crash on next start.

_Signature:_

```
void set_has_vendor_device (session ref session_ref, VM ref self, bool value)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |The VM on which to set this flag        |
|`bool`                        |`value`                       |True to provide the vendor PCI device.  |

_Minimum Role:_ vm-admin

_Return Type:_ `void`

### RPC name: set&#95;HVM&#95;boot&#95;params

_Overview:_

Set the HVM/boot&#95;params field of the given VM.

_Signature:_

```
void set_HVM_boot_params (session ref session_ref, VM ref self, (string -> string) map value)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |reference to the object                 |
|`(string -> string) map`      |`value`                       |New value to set                        |

_Minimum Role:_ vm-admin

_Return Type:_ `void`

### RPC name: set&#95;HVM&#95;boot&#95;policy

**This message is deprecated.**

_Overview:_

Set the VM.HVM&#95;boot&#95;policy field of the given VM, which will take effect when it is next started

_Signature:_

```
void set_HVM_boot_policy (session ref session_ref, VM ref self, string value)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |The VM                                  |
|`string`                      |`value`                       |The new HVM boot policy                 |

_Minimum Role:_ vm-admin

_Return Type:_ `void`

### RPC name: set&#95;HVM&#95;shadow&#95;multiplier

_Overview:_

Set the shadow memory multiplier on a halted VM

_Signature:_

```
void set_HVM_shadow_multiplier (session ref session_ref, VM ref self, float value)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |The VM                                  |
|`float`                       |`value`                       |The new shadow memory multiplier to set |

_Minimum Role:_ vm-power-admin

_Return Type:_ `void`

### RPC name: set&#95;is&#95;a&#95;template

_Overview:_

Set the is&#95;a&#95;template field of the given VM.

_Signature:_

```
void set_is_a_template (session ref session_ref, VM ref self, bool value)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |reference to the object                 |
|`bool`                        |`value`                       |New value to set                        |

_Minimum Role:_ vm-admin

_Return Type:_ `void`

### RPC name: set&#95;memory

_Overview:_

Set the memory allocation of this VM. Sets all of memory&#95;static&#95;max, memory&#95;dynamic&#95;min, and memory&#95;dynamic&#95;max to the given value, and leaves memory&#95;static&#95;min untouched.

_Signature:_

```
void set_memory (session ref session_ref, VM ref self, int value)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |The VM                                  |
|`int`                         |`value`                       |The new memory allocation &#40;bytes&#41;.|

_Minimum Role:_ vm-power-admin

_Return Type:_ `void`

### RPC name: set&#95;memory&#95;dynamic&#95;max

_Overview:_

Set the value of the memory&#95;dynamic&#95;max field

_Signature:_

```
void set_memory_dynamic_max (session ref session_ref, VM ref self, int value)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |The VM to modify                        |
|`int`                         |`value`                       |The new value of memory&#95;dynamic&#95;max|

_Minimum Role:_ vm-power-admin

_Return Type:_ `void`

### RPC name: set&#95;memory&#95;dynamic&#95;min

_Overview:_

Set the value of the memory&#95;dynamic&#95;min field

_Signature:_

```
void set_memory_dynamic_min (session ref session_ref, VM ref self, int value)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |The VM to modify                        |
|`int`                         |`value`                       |The new value of memory&#95;dynamic&#95;min|

_Minimum Role:_ vm-power-admin

_Return Type:_ `void`

### RPC name: set&#95;memory&#95;dynamic&#95;range

_Overview:_

Set the minimum and maximum amounts of physical memory the VM is allowed to use.

_Signature:_

```
void set_memory_dynamic_range (session ref session_ref, VM ref self, int min, int max)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |The VM                                  |
|`int`                         |`min`                         |The new minimum value                   |
|`int`                         |`max`                         |The new maximum value                   |

_Minimum Role:_ vm-power-admin

_Return Type:_ `void`

### RPC name: set&#95;memory&#95;limits

_Overview:_

Set the memory limits of this VM.

_Signature:_

```
void set_memory_limits (session ref session_ref, VM ref self, int static_min, int static_max, int dynamic_min, int dynamic_max)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |The VM                                  |
|`int`                         |`static_min`                  |The new value of memory&#95;static&#95;min.|
|`int`                         |`static_max`                  |The new value of memory&#95;static&#95;max.|
|`int`                         |`dynamic_min`                 |The new value of memory&#95;dynamic&#95;min.|
|`int`                         |`dynamic_max`                 |The new value of memory&#95;dynamic&#95;max.|

_Minimum Role:_ vm-power-admin

_Return Type:_ `void`

### RPC name: set&#95;memory&#95;static&#95;max

_Overview:_

Set the value of the memory&#95;static&#95;max field

_Signature:_

```
void set_memory_static_max (session ref session_ref, VM ref self, int value)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |The VM to modify                        |
|`int`                         |`value`                       |The new value of memory&#95;static&#95;max|

_Minimum Role:_ vm-power-admin

_Return Type:_ `void`

_Possible Error Codes:_ `HA_OPERATION_WOULD_BREAK_FAILOVER_PLAN`

### RPC name: set&#95;memory&#95;static&#95;min

_Overview:_

Set the value of the memory&#95;static&#95;min field

_Signature:_

```
void set_memory_static_min (session ref session_ref, VM ref self, int value)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |The VM to modify                        |
|`int`                         |`value`                       |The new value of memory&#95;static&#95;min|

_Minimum Role:_ vm-power-admin

_Return Type:_ `void`

### RPC name: set&#95;memory&#95;static&#95;range

_Overview:_

Set the static &#40;ie boot&#45;time&#41; range of virtual memory that the VM is allowed to use.

_Signature:_

```
void set_memory_static_range (session ref session_ref, VM ref self, int min, int max)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |The VM                                  |
|`int`                         |`min`                         |The new minimum value                   |
|`int`                         |`max`                         |The new maximum value                   |

_Minimum Role:_ vm-power-admin

_Return Type:_ `void`

### RPC name: set&#95;memory&#95;target&#95;live

**This message is deprecated.**

_Overview:_

Set the memory target for a running VM

_Signature:_

```
void set_memory_target_live (session ref session_ref, VM ref self, int target)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |The VM                                  |
|`int`                         |`target`                      |The target in bytes                     |

_Minimum Role:_ vm-power-admin

_Return Type:_ `void`

### RPC name: set&#95;name&#95;description

_Overview:_

Set the name/description field of the given VM.

_Signature:_

```
void set_name_description (session ref session_ref, VM ref self, string value)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |reference to the object                 |
|`string`                      |`value`                       |New value to set                        |

_Minimum Role:_ vm-admin

_Return Type:_ `void`

### RPC name: set&#95;name&#95;label

_Overview:_

Set the name/label field of the given VM.

_Signature:_

```
void set_name_label (session ref session_ref, VM ref self, string value)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |reference to the object                 |
|`string`                      |`value`                       |New value to set                        |

_Minimum Role:_ vm-admin

_Return Type:_ `void`

### RPC name: set&#95;NVRAM

_Overview:_

_Signature:_

```
void set_NVRAM (session ref session_ref, VM ref self, (string -> string) map value)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |The VM                                  |
|`(string -> string) map`      |`value`                       |The value                               |

_Minimum Role:_ vm-admin

_Return Type:_ `void`

### RPC name: set&#95;order

_Overview:_

Set this VM's boot order

_Signature:_

```
void set_order (session ref session_ref, VM ref self, int value)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |The VM                                  |
|`int`                         |`value`                       |This VM's boot order                    |

_Minimum Role:_ pool-operator

_Return Type:_ `void`

### RPC name: set&#95;other&#95;config

_Overview:_

Set the other&#95;config field of the given VM.

_Signature:_

```
void set_other_config (session ref session_ref, VM ref self, (string -> string) map value)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |reference to the object                 |
|`(string -> string) map`      |`value`                       |New value to set                        |

_Minimum Role:_ vm-admin

_Return Type:_ `void`

### RPC name: set&#95;PCI&#95;bus

**This message is deprecated.**

_Overview:_

Set the PCI&#95;bus field of the given VM.

_Signature:_

```
void set_PCI_bus (session ref session_ref, VM ref self, string value)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |reference to the object                 |
|`string`                      |`value`                       |New value to set                        |

_Minimum Role:_ vm-admin

_Return Type:_ `void`

### RPC name: set&#95;platform

_Overview:_

Set the platform field of the given VM.

_Signature:_

```
void set_platform (session ref session_ref, VM ref self, (string -> string) map value)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |reference to the object                 |
|`(string -> string) map`      |`value`                       |New value to set                        |

_Minimum Role:_ vm-admin

_Return Type:_ `void`

### RPC name: set&#95;protection&#95;policy

**This message is removed.**

_Overview:_

Set the value of the protection&#95;policy field

_Signature:_

```
void set_protection_policy (session ref session_ref, VM ref self, VMPP ref value)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |The VM                                  |
|`VMPP ref`                    |`value`                       |The value                               |

_Minimum Role:_ pool-operator

_Return Type:_ `void`

### RPC name: set&#95;PV&#95;args

_Overview:_

Set the PV/args field of the given VM.

_Signature:_

```
void set_PV_args (session ref session_ref, VM ref self, string value)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |reference to the object                 |
|`string`                      |`value`                       |New value to set                        |

_Minimum Role:_ vm-admin

_Return Type:_ `void`

### RPC name: set&#95;PV&#95;bootloader

_Overview:_

Set the PV/bootloader field of the given VM.

_Signature:_

```
void set_PV_bootloader (session ref session_ref, VM ref self, string value)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |reference to the object                 |
|`string`                      |`value`                       |New value to set                        |

_Minimum Role:_ vm-admin

_Return Type:_ `void`

### RPC name: set&#95;PV&#95;bootloader&#95;args

_Overview:_

Set the PV/bootloader&#95;args field of the given VM.

_Signature:_

```
void set_PV_bootloader_args (session ref session_ref, VM ref self, string value)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |reference to the object                 |
|`string`                      |`value`                       |New value to set                        |

_Minimum Role:_ vm-admin

_Return Type:_ `void`

### RPC name: set&#95;PV&#95;kernel

_Overview:_

Set the PV/kernel field of the given VM.

_Signature:_

```
void set_PV_kernel (session ref session_ref, VM ref self, string value)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |reference to the object                 |
|`string`                      |`value`                       |New value to set                        |

_Minimum Role:_ vm-admin

_Return Type:_ `void`

### RPC name: set&#95;PV&#95;legacy&#95;args

_Overview:_

Set the PV/legacy&#95;args field of the given VM.

_Signature:_

```
void set_PV_legacy_args (session ref session_ref, VM ref self, string value)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |reference to the object                 |
|`string`                      |`value`                       |New value to set                        |

_Minimum Role:_ vm-admin

_Return Type:_ `void`

### RPC name: set&#95;PV&#95;ramdisk

_Overview:_

Set the PV/ramdisk field of the given VM.

_Signature:_

```
void set_PV_ramdisk (session ref session_ref, VM ref self, string value)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |reference to the object                 |
|`string`                      |`value`                       |New value to set                        |

_Minimum Role:_ vm-admin

_Return Type:_ `void`

### RPC name: set&#95;recommendations

_Overview:_

Set the recommendations field of the given VM.

_Signature:_

```
void set_recommendations (session ref session_ref, VM ref self, string value)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |reference to the object                 |
|`string`                      |`value`                       |New value to set                        |

_Minimum Role:_ vm-admin

_Return Type:_ `void`

### RPC name: set&#95;shadow&#95;multiplier&#95;live

_Overview:_

Set the shadow memory multiplier on a running VM

_Signature:_

```
void set_shadow_multiplier_live (session ref session_ref, VM ref self, float multiplier)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |The VM                                  |
|`float`                       |`multiplier`                  |The new shadow memory multiplier to set |

_Minimum Role:_ vm-power-admin

_Return Type:_ `void`

### RPC name: set&#95;shutdown&#95;delay

_Overview:_

Set this VM's shutdown delay in seconds

_Signature:_

```
void set_shutdown_delay (session ref session_ref, VM ref self, int value)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |The VM                                  |
|`int`                         |`value`                       |This VM's shutdown delay in seconds     |

_Minimum Role:_ pool-operator

_Return Type:_ `void`

### RPC name: set&#95;snapshot&#95;schedule

_Overview:_

Set the value of the snapshot schedule field

_Signature:_

```
void set_snapshot_schedule (session ref session_ref, VM ref self, VMSS ref value)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |The VM                                  |
|`VMSS ref`                    |`value`                       |The value                               |

_Minimum Role:_ pool-operator

_Return Type:_ `void`

### RPC name: set&#95;start&#95;delay

_Overview:_

Set this VM's start delay in seconds

_Signature:_

```
void set_start_delay (session ref session_ref, VM ref self, int value)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |The VM                                  |
|`int`                         |`value`                       |This VM's start delay in seconds        |

_Minimum Role:_ pool-operator

_Return Type:_ `void`

### RPC name: set&#95;suspend&#95;SR

_Overview:_

Set the suspend&#95;SR field of the given VM.

_Signature:_

```
void set_suspend_SR (session ref session_ref, VM ref self, SR ref value)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |reference to the object                 |
|`SR ref`                      |`value`                       |New value to set                        |

_Minimum Role:_ vm-admin

_Return Type:_ `void`

### RPC name: set&#95;suspend&#95;VDI

_Overview:_

Set this VM's suspend VDI, which must be indentical to its current one

_Signature:_

```
void set_suspend_VDI (session ref session_ref, VM ref self, VDI ref value)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |The VM                                  |
|`VDI ref`                     |`value`                       |The suspend VDI uuid                    |

_Minimum Role:_ pool-operator

_Return Type:_ `void`

### RPC name: set&#95;tags

_Overview:_

Set the tags field of the given VM.

_Signature:_

```
void set_tags (session ref session_ref, VM ref self, string set value)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |reference to the object                 |
|`string set`                  |`value`                       |New value to set                        |

_Minimum Role:_ vm-operator

_Return Type:_ `void`

### RPC name: set&#95;uefi&#95;mode

_Overview:_

Set the UEFI mode of a VM

_Signature:_

```
string set_uefi_mode (session ref session_ref, VM ref self, vm_uefi_mode mode)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |The VM                                  |
|`vm_uefi_mode`                |`mode`                        |The UEFI mode to set                    |

_Minimum Role:_ pool-admin

_Return Type:_ `string`

Result from the varstore&#45;sb&#45;state call

### RPC name: set&#95;user&#95;version

_Overview:_

Set the user&#95;version field of the given VM.

_Signature:_

```
void set_user_version (session ref session_ref, VM ref self, int value)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |reference to the object                 |
|`int`                         |`value`                       |New value to set                        |

_Minimum Role:_ vm-admin

_Return Type:_ `void`

### RPC name: set&#95;VCPUs&#95;at&#95;startup

_Overview:_

Set the number of startup VCPUs for a halted VM

_Signature:_

```
void set_VCPUs_at_startup (session ref session_ref, VM ref self, int value)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |The VM                                  |
|`int`                         |`value`                       |The new maximum number of VCPUs         |

_Minimum Role:_ vm-admin

_Return Type:_ `void`

### RPC name: set&#95;VCPUs&#95;max

_Overview:_

Set the maximum number of VCPUs for a halted VM

_Signature:_

```
void set_VCPUs_max (session ref session_ref, VM ref self, int value)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |The VM                                  |
|`int`                         |`value`                       |The new maximum number of VCPUs         |

_Minimum Role:_ vm-admin

_Return Type:_ `void`

### RPC name: set&#95;VCPUs&#95;number&#95;live

_Overview:_

Set the number of VCPUs for a running VM

_Signature:_

```
void set_VCPUs_number_live (session ref session_ref, VM ref self, int nvcpu)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |The VM                                  |
|`int`                         |`nvcpu`                       |The number of VCPUs                     |

_Minimum Role:_ vm-admin

_Return Type:_ `void`

_Possible Error Codes:_ `OPERATION_NOT_ALLOWED`, `LICENCE_RESTRICTION`

### RPC name: set&#95;VCPUs&#95;params

_Overview:_

Set the VCPUs/params field of the given VM.

_Signature:_

```
void set_VCPUs_params (session ref session_ref, VM ref self, (string -> string) map value)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |reference to the object                 |
|`(string -> string) map`      |`value`                       |New value to set                        |

_Minimum Role:_ vm-admin

_Return Type:_ `void`

### RPC name: set&#95;xenstore&#95;data

_Overview:_

Set the xenstore&#95;data field of the given VM.

_Signature:_

```
void set_xenstore_data (session ref session_ref, VM ref self, (string -> string) map value)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |reference to the object                 |
|`(string -> string) map`      |`value`                       |New value to set                        |

_Minimum Role:_ vm-admin

_Return Type:_ `void`

### RPC name: shutdown

_Overview:_

Attempts to first clean shutdown a VM and if it should fail then perform a hard shutdown on it.

_Signature:_

```
void shutdown (session ref session_ref, VM ref vm)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`vm`                          |The VM to shutdown                      |

_Minimum Role:_ client-cert

_Return Type:_ `void`

_Possible Error Codes:_ `VM_BAD_POWER_STATE`, `OTHER_OPERATION_IN_PROGRESS`, `OPERATION_NOT_ALLOWED`, `VM_IS_TEMPLATE`

### RPC name: snapshot

_Overview:_

Snapshots the specified VM, making a new VM. Snapshot automatically exploits the capabilities of the underlying storage repository in which the VM's disk images are stored &#40;e.g. Copy on Write&#41;.

_Signature:_

```
VM ref snapshot (session ref session_ref, VM ref vm, string new_name, VDI ref set ignore_vdis)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`vm`                          |The VM to be snapshotted                |
|`string`                      |`new_name`                    |The name of the snapshotted VM          |
|`VDI ref set`                 |`ignore_vdis`                 |A list of VDIs to ignore for the snapshot|

_Minimum Role:_ vm-power-admin

_Return Type:_ `VM ref`

The reference of the newly created VM.

_Possible Error Codes:_ `VM_BAD_POWER_STATE`, `SR_FULL`, `OPERATION_NOT_ALLOWED`

### RPC name: snapshot&#95;with&#95;quiesce

**This message is removed.**

_Overview:_

Snapshots the specified VM with quiesce, making a new VM. Snapshot automatically exploits the capabilities of the underlying storage repository in which the VM's disk images are stored &#40;e.g. Copy on Write&#41;.

_Signature:_

```
VM ref snapshot_with_quiesce (session ref session_ref, VM ref vm, string new_name)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`vm`                          |The VM to be snapshotted                |
|`string`                      |`new_name`                    |The name of the snapshotted VM          |

_Minimum Role:_ vm-power-admin

_Return Type:_ `VM ref`

The reference of the newly created VM.

_Possible Error Codes:_ `VM_BAD_POWER_STATE`, `SR_FULL`, `OPERATION_NOT_ALLOWED`, `VM_SNAPSHOT_WITH_QUIESCE_FAILED`, `VM_SNAPSHOT_WITH_QUIESCE_TIMEOUT`, `VM_SNAPSHOT_WITH_QUIESCE_PLUGIN_DEOS_NOT_RESPOND`, `VM_SNAPSHOT_WITH_QUIESCE_NOT_SUPPORTED`

### RPC name: start

_Overview:_

Start the specified VM.  This function can only be called with the VM is in the Halted State.

_Signature:_

```
void start (session ref session_ref, VM ref vm, bool start_paused, bool force)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`vm`                          |The VM to start                         |
|`bool`                        |`start_paused`                |Instantiate VM in paused state if set to true.|
|`bool`                        |`force`                       |Attempt to force the VM to start. If this flag is false then the VM may fail pre&#45;boot safety checks &#40;e.g. if the CPU the VM last booted on looks substantially different to the current one&#41;|

_Minimum Role:_ vm-operator

_Return Type:_ `void`

_Possible Error Codes:_ `VM_BAD_POWER_STATE`, `VM_HVM_REQUIRED`, `VM_IS_TEMPLATE`, `OTHER_OPERATION_IN_PROGRESS`, `OPERATION_NOT_ALLOWED`, `BOOTLOADER_FAILED`, `UNKNOWN_BOOTLOADER`, `NO_HOSTS_AVAILABLE`, `LICENCE_RESTRICTION`

### RPC name: start&#95;on

_Overview:_

Start the specified VM on a particular host.  This function can only be called with the VM is in the Halted State.

_Signature:_

```
void start_on (session ref session_ref, VM ref vm, host ref host, bool start_paused, bool force)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`vm`                          |The VM to start                         |
|`host ref`                    |`host`                        |The Host on which to start the VM       |
|`bool`                        |`start_paused`                |Instantiate VM in paused state if set to true.|
|`bool`                        |`force`                       |Attempt to force the VM to start. If this flag is false then the VM may fail pre&#45;boot safety checks &#40;e.g. if the CPU the VM last booted on looks substantially different to the current one&#41;|

_Minimum Role:_ client-cert

_Return Type:_ `void`

_Possible Error Codes:_ `VM_BAD_POWER_STATE`, `VM_IS_TEMPLATE`, `OTHER_OPERATION_IN_PROGRESS`, `OPERATION_NOT_ALLOWED`, `BOOTLOADER_FAILED`, `UNKNOWN_BOOTLOADER`

### RPC name: suspend

_Overview:_

Suspend the specified VM to disk.  This can only be called when the specified VM is in the Running state.

_Signature:_

```
void suspend (session ref session_ref, VM ref vm)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`vm`                          |The VM to suspend                       |

_Minimum Role:_ client-cert

_Return Type:_ `void`

_Possible Error Codes:_ `VM_BAD_POWER_STATE`, `OTHER_OPERATION_IN_PROGRESS`, `OPERATION_NOT_ALLOWED`, `VM_IS_TEMPLATE`

### RPC name: unpause

_Overview:_

Resume the specified VM. This can only be called when the specified VM is in the Paused state.

_Signature:_

```
void unpause (session ref session_ref, VM ref vm)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`vm`                          |The VM to unpause                       |

_Minimum Role:_ vm-operator

_Return Type:_ `void`

_Possible Error Codes:_ `VM_BAD_POWER_STATE`, `OPERATION_NOT_ALLOWED`, `VM_IS_TEMPLATE`

### RPC name: update&#95;allowed&#95;operations

_Overview:_

Recomputes the list of acceptable operations

_Signature:_

```
void update_allowed_operations (session ref session_ref, VM ref self)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |reference to the object                 |

_Minimum Role:_ pool-admin

_Return Type:_ `void`

### RPC name: wait&#95;memory&#95;target&#95;live

**This message is deprecated.**

_Overview:_

Wait for a running VM to reach its current memory target

_Signature:_

```
void wait_memory_target_live (session ref session_ref, VM ref self)
```

_Arguments:_

|Type                          |Name                          |Description                             |
|:-----------------------------|:-----------------------------|:---------------------------------------|
|`session ref`                 |`session_ref`                 |Reference to a valid session            |
|`VM ref`                      |`self`                        |The VM                                  |

_Minimum Role:_ read-only

_Return Type:_ `void`
